@@ -2,13 +2,13 @@
 
 struct _AdwDemoPreferencesWindow
 {
-  AdwPreferencesWindow parent_instance;
+  AdwPreferencesDialog parent_instance;
 
-  GtkWidget *subpage1;
-  GtkWidget *subpage2;
+  AdwNavigationPage *subpage1;
+  AdwNavigationPage *subpage2;
 };
 
-G_DEFINE_FINAL_TYPE (AdwDemoPreferencesWindow, adw_demo_preferences_window, ADW_TYPE_PREFERENCES_WINDOW)
+G_DEFINE_FINAL_TYPE (AdwDemoPreferencesWindow, adw_demo_preferences_window, ADW_TYPE_PREFERENCES_DIALOG)
 
 AdwDemoPreferencesWindow *
 adw_demo_preferences_window_new (void)
@@ -16,28 +16,22 @@ adw_demo_preferences_window_new (void)
   return g_object_new (ADW_TYPE_DEMO_PREFERENCES_WINDOW, NULL);
 }
 
-static void
-return_to_preferences_cb (AdwDemoPreferencesWindow *self)
-{
-  adw_preferences_window_close_subpage (ADW_PREFERENCES_WINDOW (self));
-}
-
 static void
 subpage1_activated_cb (AdwDemoPreferencesWindow *self)
 {
-  adw_preferences_window_present_subpage (ADW_PREFERENCES_WINDOW (self), self->subpage1);
+  adw_preferences_dialog_push_subpage (ADW_PREFERENCES_DIALOG (self), self->subpage1);
 }
 
 static void
 subpage2_activated_cb (AdwDemoPreferencesWindow *self)
 {
-  adw_preferences_window_present_subpage (ADW_PREFERENCES_WINDOW (self), self->subpage2);
+  adw_preferences_dialog_push_subpage (ADW_PREFERENCES_DIALOG (self), self->subpage2);
 }
 
 static void
-toast_show_cb (AdwPreferencesWindow *window)
+toast_show_cb (AdwPreferencesDialog *window)
 {
-  adw_preferences_window_add_toast (window, adw_toast_new ("Example Toast"));
+  adw_preferences_dialog_add_toast (window, adw_toast_new ("Example Toast"));
 }
 
 static void
@@ -50,7 +44,6 @@ adw_demo_preferences_window_class_init (AdwDemoPreferencesWindowClass *klass)
   gtk_widget_class_bind_template_child (widget_class, AdwDemoPreferencesWindow, subpage1);
   gtk_widget_class_bind_template_child (widget_class, AdwDemoPreferencesWindow, subpage2);
 
-  gtk_widget_class_bind_template_callback (widget_class, return_to_preferences_cb);
   gtk_widget_class_bind_template_callback (widget_class, subpage1_activated_cb);
   gtk_widget_class_bind_template_callback (widget_class, subpage2_activated_cb);
 
