@@ -4,10 +4,10 @@
 
 struct _AdwViewSwitcherDemoWindow
 {
-  AdwWindow parent_instance;
+  AdwDialog parent_instance;
 };
 
-G_DEFINE_FINAL_TYPE (AdwViewSwitcherDemoWindow, adw_view_switcher_demo_window, ADW_TYPE_WINDOW)
+G_DEFINE_FINAL_TYPE (AdwViewSwitcherDemoWindow, adw_view_switcher_demo_window, ADW_TYPE_DIALOG)
 
 static void
 adw_view_switcher_demo_window_class_init (AdwViewSwitcherDemoWindowClass *klass)
