@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2020 Alexander Mikhaylenko <alexm@gnome.org>
+ * Copyright (C) 2020 Alice Mikhaylenko <alicem@gnome.org>
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
  */
@@ -14,6 +14,9 @@
 
 #include <gtk/gtk.h>
 
+#include "adw-breakpoint.h"
+#include "adw-dialog.h"
+
 G_BEGIN_DECLS
 
 #define ADW_TYPE_APPLICATION_WINDOW (adw_application_window_get_type())
@@ -38,4 +41,17 @@ void       adw_application_window_set_content (AdwApplicationWindow *self,
 ADW_AVAILABLE_IN_ALL
 GtkWidget *adw_application_window_get_content (AdwApplicationWindow *self);
 
+ADW_AVAILABLE_IN_1_4
+void adw_application_window_add_breakpoint (AdwApplicationWindow *self,
+                                            AdwBreakpoint        *breakpoint);
+
+ADW_AVAILABLE_IN_1_4
+AdwBreakpoint *adw_application_window_get_current_breakpoint (AdwApplicationWindow *self);
+
+ADW_AVAILABLE_IN_1_5
+GListModel *adw_application_window_get_dialogs (AdwApplicationWindow *self);
+
+ADW_AVAILABLE_IN_1_5
+AdwDialog *adw_application_window_get_visible_dialog (AdwApplicationWindow *self);
+
 G_END_DECLS
