@@ -3,7 +3,7 @@
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * Author: Alexander Mikhaylenko <alexander.mikhaylenko@puri.sm>
+ * Author: Alice Mikhaylenko <alice.mikhaylenko@puri.sm>
  */
 
 #include "config.h"
@@ -26,8 +26,6 @@
  *
  * It is useful for deriving subclasses, since it provides common code needed
  * for handling a single child widget.
- *
- * Since: 1.0
  */
 
 typedef struct
@@ -112,15 +110,11 @@ adw_bin_class_init (AdwBinClass *klass)
    * AdwBin:child: (attributes org.gtk.Property.get=adw_bin_get_child org.gtk.Property.set=adw_bin_set_child)
    *
    * The child widget of the `AdwBin`.
-   *
-   * Since: 1.0
    */
   props[PROP_CHILD] =
-      g_param_spec_object ("child",
-                           "Child",
-                           "The child widget",
-                           GTK_TYPE_WIDGET,
-                           G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+    g_param_spec_object ("child", NULL, NULL,
+                         GTK_TYPE_WIDGET,
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
@@ -158,8 +152,6 @@ adw_bin_buildable_init (GtkBuildableIface *iface)
  * Creates a new `AdwBin`.
  *
  * Returns: the new created `AdwBin`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_bin_new (void)
@@ -174,8 +166,6 @@ adw_bin_new (void)
  * Gets the child widget of @self.
  *
  * Returns: (nullable) (transfer none): the child widget of @self
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_bin_get_child (AdwBin *self)
@@ -195,8 +185,6 @@ adw_bin_get_child (AdwBin *self)
  * @child: (nullable): the child widget
  *
  * Sets the child widget of @self.
- *
- * Since: 1.0
  */
 void
 adw_bin_set_child (AdwBin    *self,
@@ -207,6 +195,9 @@ adw_bin_set_child (AdwBin    *self,
   g_return_if_fail (ADW_IS_BIN (self));
   g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
 
+  if (child)
+    g_return_if_fail (gtk_widget_get_parent (child) == NULL);
+
   priv = adw_bin_get_instance_private (self);
 
   if (priv->child == child)
