@@ -14,6 +14,8 @@
 
 #include <gtk/gtk.h>
 
+#include "adw-length-unit.h"
+
 G_BEGIN_DECLS
 
 #define ADW_TYPE_CLAMP_LAYOUT (adw_clamp_layout_get_type())
@@ -36,4 +38,10 @@ ADW_AVAILABLE_IN_ALL
 void adw_clamp_layout_set_tightening_threshold (AdwClampLayout *self,
                                                 int             tightening_threshold);
 
+ADW_AVAILABLE_IN_ALL
+AdwLengthUnit adw_clamp_layout_get_unit (AdwClampLayout *self);
+ADW_AVAILABLE_IN_ALL
+void          adw_clamp_layout_set_unit (AdwClampLayout *self,
+                                         AdwLengthUnit   unit);
+
 G_END_DECLS
