@@ -12,7 +12,6 @@
 
 #include "adw-animation-util.h"
 #include "adw-gizmo-private.h"
-#include "adw-macros-private.h"
 #include "adw-shadow-helper-private.h"
 #include "adw-spring-animation.h"
 #include "adw-swipeable.h"
@@ -81,7 +80,7 @@
  * `AdwFlap` has a single CSS node with name `flap`. The node will get the style
  * classes `.folded` when it is folded, and `.unfolded` when it's not.
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 
 /**
@@ -94,7 +93,7 @@
  *
  * Describes the possible folding behavior of a [class@Flap] widget.
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 
 /**
@@ -113,9 +112,7 @@
  * [class@Flap] widget, as well as which areas can be swiped via
  * [property@Flap:swipe-to-open] and [property@Flap:swipe-to-close].
  *
- * New values may be added to this enum over time.
- *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 
 typedef struct {
@@ -331,7 +328,7 @@ animate_reveal (AdwFlap *self,
                                        self->reveal_progress);
   adw_spring_animation_set_value_to (ADW_SPRING_ANIMATION (self->reveal_animation), to);
 
-  if (!G_APPROX_VALUE (self->reveal_progress, to, FLT_EPSILON))
+  if (!G_APPROX_VALUE (self->reveal_progress, to, DBL_EPSILON))
     adw_spring_animation_set_initial_velocity (ADW_SPRING_ANIMATION (self->reveal_animation),
                                                velocity / adw_swipeable_get_distance (ADW_SWIPEABLE (self)));
   else
@@ -363,8 +360,6 @@ static void
 set_folded (AdwFlap  *self,
             gboolean  folded)
 {
-  GtkStyleContext *context;
-
   folded = !!folded;
 
   if (self->folded == folded)
@@ -386,13 +381,12 @@ set_folded (AdwFlap  *self,
   if (!self->locked)
     set_reveal_flap (self, !self->folded, 0);
 
-  context = gtk_widget_get_style_context (GTK_WIDGET (self));
   if (folded) {
-    gtk_style_context_add_class (context, "folded");
-    gtk_style_context_remove_class (context, "unfolded");
+    gtk_widget_add_css_class (GTK_WIDGET (self), "folded");
+    gtk_widget_remove_css_class (GTK_WIDGET (self), "unfolded");
   } else {
-    gtk_style_context_remove_class (context, "folded");
-    gtk_style_context_add_class (context, "unfolded");
+    gtk_widget_remove_css_class (GTK_WIDGET (self), "folded");
+    gtk_widget_add_css_class (GTK_WIDGET (self), "unfolded");
   }
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_FOLDED]);
@@ -412,10 +406,14 @@ static void
 begin_swipe_cb (AdwSwipeTracker *tracker,
                 AdwFlap         *self)
 {
-  if (self->reveal_progress <= 0 && !self->swipe_to_open)
+  if ((G_APPROX_VALUE (self->reveal_progress, 0, DBL_EPSILON) ||
+       self->reveal_progress < 0) &&
+      !self->swipe_to_open)
     return;
 
-  if (self->reveal_progress >= 1 && !self->swipe_to_close)
+  if ((G_APPROX_VALUE (self->reveal_progress, 1, DBL_EPSILON) ||
+       self->reveal_progress > 1) &&
+      !self->swipe_to_close)
     return;
 
   adw_animation_pause (self->reveal_animation);
@@ -691,9 +689,10 @@ interpolate_reveal (AdwFlap  *self,
                     int      *content_size,
                     int      *separator_size)
 {
-  if (self->reveal_progress <= 0) {
+  
+  if (G_APPROX_VALUE (self->reveal_progress, 0, DBL_EPSILON) || self->reveal_progress < 0) {
     compute_sizes (self, width, height, folded, FALSE, flap_size, content_size, separator_size);
-  } else if (self->reveal_progress >= 1) {
+  } else if (G_APPROX_VALUE (self->reveal_progress, 1, DBL_EPSILON) || self->reveal_progress > 1) {
     compute_sizes (self, width, height, folded, TRUE, flap_size, content_size, separator_size);
   } else {
     int flap_revealed, content_revealed, separator_revealed;
@@ -722,9 +721,9 @@ interpolate_fold (AdwFlap *self,
                   int     *content_size,
                   int     *separator_size)
 {
-  if (self->fold_progress <= 0) {
+  if (G_APPROX_VALUE (self->fold_progress, 0, DBL_EPSILON) || self->fold_progress < 0) {
     interpolate_reveal (self, width, height, FALSE, flap_size, content_size, separator_size);
-  } else if (self->fold_progress >= 1) {
+  } else if (G_APPROX_VALUE (self->fold_progress, 1, DBL_EPSILON) || self->fold_progress > 1) {
     interpolate_reveal (self, width, height, TRUE, flap_size, content_size, separator_size);
   } else {
     int flap_folded, content_folded, separator_folded;
@@ -988,7 +987,7 @@ adw_flap_measure (GtkWidget      *widget,
     }
 
     min = MAX (content_min + (int) round ((flap_min + separator_min) * min_progress), flap_min);
-    nat = MAX (content_nat + (int) round ((flap_nat + separator_min) * nat_progress), flap_nat);
+    nat = MAX (content_nat + (int) round ((flap_nat + separator_nat) * nat_progress), flap_nat);
   } else {
     min = MAX (MAX (content_min, flap_min), separator_min);
     nat = MAX (MAX (content_nat, flap_nat), separator_nat);
@@ -1230,10 +1229,9 @@ adw_flap_dispose (GObject *object)
 {
   AdwFlap *self = ADW_FLAP (object);
 
-  adw_flap_set_flap (self, NULL);
-  adw_flap_set_separator (self, NULL);
-  adw_flap_set_content (self, NULL);
-
+  g_clear_pointer (&self->flap.widget, gtk_widget_unparent);
+  g_clear_pointer (&self->separator.widget, gtk_widget_unparent);
+  g_clear_pointer (&self->content.widget, gtk_widget_unparent);
   g_clear_pointer (&self->shield, gtk_widget_unparent);
 
   g_clear_object (&self->shadow_helper);
@@ -1270,12 +1268,12 @@ adw_flap_class_init (AdwFlapClass *klass)
    *
    * It's always displayed when unfolded, and partially visible when folded.
    *
-   * Since: 1.0
+   * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
    */
   props[PROP_CONTENT] =
     g_param_spec_object ("content", NULL, NULL,
                          GTK_TYPE_WIDGET,
-                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_DEPRECATED);
 
   /**
    * AdwFlap:flap: (attributes org.gtk.Property.get=adw_flap_get_flap org.gtk.Property.set=adw_flap_set_flap)
@@ -1284,12 +1282,12 @@ adw_flap_class_init (AdwFlapClass *klass)
    *
    * It's only visible when [property@Flap:reveal-progress] is greater than 0.
    *
-   * Since: 1.0
+   * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
    */
   props[PROP_FLAP] =
     g_param_spec_object ("flap", NULL, NULL,
                          GTK_TYPE_WIDGET,
-                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_DEPRECATED);
 
   /**
    * AdwFlap:separator: (attributes org.gtk.Property.get=adw_flap_get_separator org.gtk.Property.set=adw_flap_set_separator)
@@ -1300,12 +1298,12 @@ adw_flap_class_init (AdwFlapClass *klass)
    * When exactly it's visible depends on the [property@Flap:transition-type]
    * value.
    *
-   * Since: 1.0
+   * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
    */
   props[PROP_SEPARATOR] =
     g_param_spec_object ("separator", NULL, NULL,
                          GTK_TYPE_WIDGET,
-                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_DEPRECATED);
 
   /**
    * AdwFlap:flap-position: (attributes org.gtk.Property.get=adw_flap_get_flap_position org.gtk.Property.set=adw_flap_set_flap_position)
@@ -1315,25 +1313,25 @@ adw_flap_class_init (AdwFlapClass *klass)
    * If it's set to `GTK_PACK_START`, the flap is displayed before the content,
    * if `GTK_PACK_END`, it's displayed after the content.
    *
-   * Since: 1.0
+   * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
    */
   props[PROP_FLAP_POSITION] =
     g_param_spec_enum ("flap-position", NULL, NULL,
                        GTK_TYPE_PACK_TYPE,
                        GTK_PACK_START,
-                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_DEPRECATED);
 
   /**
    * AdwFlap:reveal-flap: (attributes org.gtk.Property.get=adw_flap_get_reveal_flap org.gtk.Property.set=adw_flap_set_reveal_flap)
    *
    * Whether the flap widget is revealed.
    *
-   * Since: 1.0
+   * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
    */
   props[PROP_REVEAL_FLAP] =
     g_param_spec_boolean ("reveal-flap", NULL, NULL,
                           TRUE,
-                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_DEPRECATED);
 
   /**
    * AdwFlap:reveal-params: (attributes org.gtk.Property.get=adw_flap_get_reveal_params org.gtk.Property.set=adw_flap_set_reveal_params)
@@ -1346,12 +1344,12 @@ adw_flap_class_init (AdwFlapClass *klass)
    * adw_spring_params_new (1, 0.5, 500)
    * ```
    *
-   * Since: 1.0
+   * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
    */
   props[PROP_REVEAL_PARAMS] =
     g_param_spec_boxed ("reveal-params", NULL, NULL,
                         ADW_TYPE_SPRING_PARAMS,
-                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_DEPRECATED);
 
   /**
    * AdwFlap:reveal-progress: (attributes org.gtk.Property.get=adw_flap_get_reveal_progress)
@@ -1362,25 +1360,25 @@ adw_flap_class_init (AdwFlapClass *klass)
    *
    * See [property@Flap:reveal-flap].
    *
-   * Since: 1.0
+   * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
    */
   props[PROP_REVEAL_PROGRESS] =
     g_param_spec_double ("reveal-progress", NULL, NULL,
                           0.0, 1.0, 1.0,
-                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS | G_PARAM_DEPRECATED);
 
   /**
-   * AdwFlap:fold-policy: (attributes org.gtk.Property.get=adw_flap_get_fold_policy)
+   * AdwFlap:fold-policy: (attributes org.gtk.Property.get=adw_flap_get_fold_policy org.gtk.Property.set=adw_flap_set_fold_policy)
    *
    * The fold policy for the flap.
    *
-   * Since: 1.0
+   * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
    */
   props[PROP_FOLD_POLICY] =
     g_param_spec_enum ("fold-policy", NULL, NULL,
                        ADW_TYPE_FLAP_FOLD_POLICY,
                        ADW_FLAP_FOLD_POLICY_AUTO,
-                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_DEPRECATED);
 
   /**
    * AdwFlap:fold-threshold-policy: (attributes org.gtk.Property.get=adw_flap_get_fold_threshold_policy org.gtk.Property.set=adw_flap_set_fold_threshold_policy)
@@ -1394,26 +1392,26 @@ adw_flap_class_init (AdwFlapClass *klass)
    * This can be useful if you have a long ellipsizing label and want to let it
    * ellipsize instead of immediately folding.
    *
-   * Since: 1.0
+   * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
    */
   props[PROP_FOLD_THRESHOLD_POLICY] =
     g_param_spec_enum ("fold-threshold-policy", NULL, NULL,
                        ADW_TYPE_FOLD_THRESHOLD_POLICY,
                        ADW_FOLD_THRESHOLD_POLICY_MINIMUM,
-                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_DEPRECATED);
 
   /**
    * AdwFlap:fold-duration: (attributes org.gtk.Property.get=adw_flap_get_fold_duration org.gtk.Property.set=adw_flap_set_fold_duration)
    *
    * The fold transition animation duration, in milliseconds.
    *
-   * Since: 1.0
+   * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
    */
   props[PROP_FOLD_DURATION] =
     g_param_spec_uint ("fold-duration", NULL, NULL,
                        0, G_MAXINT,
                        250,
-                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_DEPRECATED);
 
   /**
    * AdwFlap:folded: (attributes org.gtk.Property.get=adw_flap_get_folded)
@@ -1422,12 +1420,12 @@ adw_flap_class_init (AdwFlapClass *klass)
    *
    * See [property@Flap:fold-policy].
    *
-   * Since: 1.0
+   * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
    */
   props[PROP_FOLDED] =
     g_param_spec_boolean ("folded", NULL, NULL,
                           FALSE,
-                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS | G_PARAM_DEPRECATED);
 
   /**
    * AdwFlap:locked: (attributes org.gtk.Property.get=adw_flap_get_locked org.gtk.Property.set=adw_flap_set_locked)
@@ -1438,12 +1436,12 @@ adw_flap_class_init (AdwFlapClass *klass)
    * unfolding it when the flap is not revealed opens it. If `TRUE`,
    * [property@Flap:reveal-flap] value never changes on its own.
    *
-   * Since: 1.0
+   * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
    */
   props[PROP_LOCKED] =
     g_param_spec_boolean ("locked", NULL, NULL,
                           FALSE,
-                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_DEPRECATED);
 
   /**
    * AdwFlap:transition-type: (attributes org.gtk.Property.get=adw_flap_get_transition_type org.gtk.Property.set=adw_flap_set_transition_type)
@@ -1455,13 +1453,13 @@ adw_flap_class_init (AdwFlapClass *klass)
    * add the [`.background`](style-classes.html#background) style class to it if
    * this is unwanted.
    *
-   * Since: 1.0
+   * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
    */
   props[PROP_TRANSITION_TYPE] =
     g_param_spec_enum ("transition-type", NULL, NULL,
                        ADW_TYPE_FLAP_TRANSITION_TYPE,
                        ADW_FLAP_TRANSITION_TYPE_OVER,
-                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_DEPRECATED);
 
   /**
    * AdwFlap:modal: (attributes org.gtk.Property.get=adw_flap_get_modal org.gtk.Property.set=adw_flap_set_modal)
@@ -1472,12 +1470,12 @@ adw_flap_class_init (AdwFlapClass *klass)
    * pressing the <kbd>Esc</kbd> key, will close the flap. If `FALSE`, clicks
    * are passed through to the content widget.
    *
-   * Since: 1.0
+   * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
    */
   props[PROP_MODAL] =
     g_param_spec_boolean ("modal", NULL, NULL,
                           TRUE,
-                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_DEPRECATED);
 
   /**
    * AdwFlap:swipe-to-open: (attributes org.gtk.Property.get=adw_flap_get_swipe_to_open org.gtk.Property.set=adw_flap_set_swipe_to_open)
@@ -1487,12 +1485,12 @@ adw_flap_class_init (AdwFlapClass *klass)
    * The area that can be swiped depends on the [property@Flap:transition-type]
    * value.
    *
-   * Since: 1.0
+   * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
    */
   props[PROP_SWIPE_TO_OPEN] =
     g_param_spec_boolean ("swipe-to-open", NULL, NULL,
                           TRUE,
-                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_DEPRECATED);
 
   /**
    * AdwFlap:swipe-to-close: (attributes org.gtk.Property.get=adw_flap_get_swipe_to_close org.gtk.Property.set=adw_flap_set_swipe_to_close)
@@ -1502,12 +1500,12 @@ adw_flap_class_init (AdwFlapClass *klass)
    * The area that can be swiped depends on the [property@Flap:transition-type]
    * value.
    *
-   * Since: 1.0
+   * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
    */
   props[PROP_SWIPE_TO_CLOSE] =
     g_param_spec_boolean ("swipe-to-close", NULL, NULL,
                           TRUE,
-                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_DEPRECATED);
 
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
@@ -1521,7 +1519,10 @@ adw_flap_class_init (AdwFlapClass *klass)
 static gboolean
 flap_close_cb (AdwFlap *self)
 {
-  if (self->reveal_progress <= 0 || self->fold_progress <= 0)
+  if (G_APPROX_VALUE (self->reveal_progress, 0, DBL_EPSILON) ||
+      self->reveal_progress < 0 ||
+      G_APPROX_VALUE (self->fold_progress, 0, DBL_EPSILON) ||
+      self->fold_progress < 0)
     return GDK_EVENT_PROPAGATE;
 
   adw_flap_set_reveal_flap (ADW_FLAP (self), FALSE);
@@ -1532,7 +1533,6 @@ flap_close_cb (AdwFlap *self)
 static void
 adw_flap_init (AdwFlap *self)
 {
-  GtkStyleContext *context = gtk_widget_get_style_context (GTK_WIDGET (self));
   GtkEventController *gesture;
   GtkShortcut *shortcut;
   AdwAnimationTarget *target;
@@ -1582,7 +1582,7 @@ adw_flap_init (AdwFlap *self)
 
   gtk_widget_set_overflow (GTK_WIDGET (self), GTK_OVERFLOW_HIDDEN);
 
-  gtk_style_context_add_class (context, "unfolded");
+  gtk_widget_add_css_class (GTK_WIDGET (self), "unfolded");
 
   target = adw_callback_animation_target_new ((AdwAnimationTargetFunc)
                                               fold_animation_value_cb,
@@ -1734,7 +1734,8 @@ adw_flap_get_swipe_area (AdwSwipeable           *swipeable,
   content_factor = transition_get_content_motion_factor (self);
 
   if (!is_drag ||
-      (flap_factor >= 1 && content_factor >= 1) ||
+      ((G_APPROX_VALUE (flap_factor, 1, DBL_EPSILON) || flap_factor > 1) &&
+       (G_APPROX_VALUE (content_factor, 1, DBL_EPSILON) || content_factor > 1)) ||
       (self->fold_progress < 1 && flap_factor > 0)) {
     rect->x = 0;
     rect->y = 0;
@@ -1794,7 +1795,7 @@ adw_flap_swipeable_init (AdwSwipeableInterface *iface)
  *
  * Returns: the newly created `AdwFlap`
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 GtkWidget *
 adw_flap_new (void)
@@ -1810,7 +1811,7 @@ adw_flap_new (void)
  *
  * Returns: (transfer none) (nullable): the content widget for @self
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 GtkWidget *
 adw_flap_get_content (AdwFlap *self)
@@ -1829,7 +1830,7 @@ adw_flap_get_content (AdwFlap *self)
  *
  * It's always displayed when unfolded, and partially visible when folded.
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 void
 adw_flap_set_content (AdwFlap   *self,
@@ -1838,6 +1839,9 @@ adw_flap_set_content (AdwFlap   *self,
   g_return_if_fail (ADW_IS_FLAP (self));
   g_return_if_fail (content == NULL || GTK_IS_WIDGET (content));
 
+  if (content)
+    g_return_if_fail (gtk_widget_get_parent (content) == NULL);
+
   if (self->content.widget == content)
     return;
 
@@ -1862,7 +1866,7 @@ adw_flap_set_content (AdwFlap   *self,
  *
  * Returns: (transfer none) (nullable): the flap widget for @self
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 GtkWidget *
 adw_flap_get_flap (AdwFlap *self)
@@ -1881,7 +1885,7 @@ adw_flap_get_flap (AdwFlap *self)
  *
  * It's only visible when [property@Flap:reveal-progress] is greater than 0.
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 void
 adw_flap_set_flap (AdwFlap   *self,
@@ -1890,6 +1894,9 @@ adw_flap_set_flap (AdwFlap   *self,
   g_return_if_fail (ADW_IS_FLAP (self));
   g_return_if_fail (flap == NULL || GTK_IS_WIDGET (flap));
 
+  if (flap)
+    g_return_if_fail (gtk_widget_get_parent (flap) == NULL);
+
   if (self->flap.widget == flap)
     return;
 
@@ -1915,7 +1922,7 @@ adw_flap_set_flap (AdwFlap   *self,
  *
  * Returns: (transfer none) (nullable): the separator widget for @self
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 GtkWidget *
 adw_flap_get_separator (AdwFlap *self)
@@ -1936,7 +1943,7 @@ adw_flap_get_separator (AdwFlap *self)
  * When exactly it's visible depends on the [property@Flap:transition-type]
  * value.
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 void
 adw_flap_set_separator (AdwFlap   *self,
@@ -1945,6 +1952,9 @@ adw_flap_set_separator (AdwFlap   *self,
   g_return_if_fail (ADW_IS_FLAP (self));
   g_return_if_fail (separator == NULL || GTK_IS_WIDGET (separator));
 
+  if (separator)
+    g_return_if_fail (gtk_widget_get_parent (separator) == NULL);
+
   if (self->separator.widget == separator)
     return;
 
@@ -1969,7 +1979,7 @@ adw_flap_set_separator (AdwFlap   *self,
  *
  * Returns: the flap position for @self
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 GtkPackType
 adw_flap_get_flap_position (AdwFlap *self)
@@ -1989,7 +1999,7 @@ adw_flap_get_flap_position (AdwFlap *self)
  * If it's set to `GTK_PACK_START`, the flap is displayed before the content,
  * if `GTK_PACK_END`, it's displayed after the content.
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 void
 adw_flap_set_flap_position (AdwFlap     *self,
@@ -2017,7 +2027,7 @@ adw_flap_set_flap_position (AdwFlap     *self,
  *
  * Returns: `TRUE` if the flap widget is revealed
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 gboolean
 adw_flap_get_reveal_flap (AdwFlap *self)
@@ -2034,7 +2044,7 @@ adw_flap_get_reveal_flap (AdwFlap *self)
  *
  * Sets whether the flap widget is revealed for @self.
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 void
 adw_flap_set_reveal_flap (AdwFlap  *self,
@@ -2053,7 +2063,7 @@ adw_flap_set_reveal_flap (AdwFlap  *self,
  *
  * Returns: the reveal animation parameters
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 AdwSpringParams *
 adw_flap_get_reveal_params (AdwFlap *self)
@@ -2076,7 +2086,7 @@ adw_flap_get_reveal_params (AdwFlap *self)
  * adw_spring_params_new (1, 0.5, 500)
  * ```
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 void
 adw_flap_set_reveal_params (AdwFlap         *self,
@@ -2106,7 +2116,7 @@ adw_flap_set_reveal_params (AdwFlap         *self,
  *
  * Returns: the current reveal progress for @self
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 double
 adw_flap_get_reveal_progress (AdwFlap *self)
@@ -2124,7 +2134,7 @@ adw_flap_get_reveal_progress (AdwFlap *self)
  *
  * Returns: the fold policy for @self
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 AdwFlapFoldPolicy
 adw_flap_get_fold_policy (AdwFlap *self)
@@ -2141,7 +2151,7 @@ adw_flap_get_fold_policy (AdwFlap *self)
  *
  * Sets the fold policy for @self.
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 void
 adw_flap_set_fold_policy (AdwFlap           *self,
@@ -2181,7 +2191,7 @@ adw_flap_set_fold_policy (AdwFlap           *self,
  *
  * Gets the fold threshold policy for @self.
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 AdwFoldThresholdPolicy
 adw_flap_get_fold_threshold_policy (AdwFlap *self)
@@ -2205,7 +2215,7 @@ adw_flap_get_fold_threshold_policy (AdwFlap *self)
  * This can be useful if you have a long ellipsizing label and want to let it
  * ellipsize instead of immediately folding.
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 void
 adw_flap_set_fold_threshold_policy (AdwFlap                *self,
@@ -2232,7 +2242,7 @@ adw_flap_set_fold_threshold_policy (AdwFlap                *self,
  *
  * Returns: the fold transition duration
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 guint
 adw_flap_get_fold_duration (AdwFlap *self)
@@ -2249,7 +2259,7 @@ adw_flap_get_fold_duration (AdwFlap *self)
  *
  * Sets the fold transition animation duration for @self, in milliseconds.
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 void
 adw_flap_set_fold_duration (AdwFlap *self,
@@ -2275,7 +2285,7 @@ adw_flap_set_fold_duration (AdwFlap *self,
  *
  * Returns: `TRUE` if @self is currently folded
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 gboolean
 adw_flap_get_folded (AdwFlap *self)
@@ -2293,7 +2303,7 @@ adw_flap_get_folded (AdwFlap *self)
  *
  * Returns: `TRUE` if @self is locked
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 gboolean
 adw_flap_get_locked (AdwFlap *self)
@@ -2314,7 +2324,7 @@ adw_flap_get_locked (AdwFlap *self)
  * unfolding it when the flap is not revealed opens it. If `TRUE`,
  * [property@Flap:reveal-flap] value never changes on its own.
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 void
 adw_flap_set_locked (AdwFlap  *self,
@@ -2340,7 +2350,7 @@ adw_flap_set_locked (AdwFlap  *self,
  *
  * Returns: the current transition type of @self
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 AdwFlapTransitionType
 adw_flap_get_transition_type (AdwFlap *self)
@@ -2362,7 +2372,7 @@ adw_flap_get_transition_type (AdwFlap *self)
  * [`.background`](style-classes.html#background) style class to it if this is
  * unwanted.
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 void
 adw_flap_set_transition_type (AdwFlap               *self,
@@ -2392,7 +2402,7 @@ adw_flap_set_transition_type (AdwFlap               *self,
  *
  * Returns: `TRUE` if @self is modal
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 gboolean
 adw_flap_get_modal (AdwFlap *self)
@@ -2413,7 +2423,7 @@ adw_flap_get_modal (AdwFlap *self)
  * pressing the <kbd>Esc</kbd> key, will close the flap. If `FALSE`, clicks are
  * passed through to the content widget.
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 void
 adw_flap_set_modal (AdwFlap  *self,
@@ -2444,7 +2454,7 @@ adw_flap_set_modal (AdwFlap  *self,
  *
  * Returns: `TRUE` if @self can be opened with a swipe gesture
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 gboolean
 adw_flap_get_swipe_to_open (AdwFlap *self)
@@ -2464,7 +2474,7 @@ adw_flap_get_swipe_to_open (AdwFlap *self)
  * The area that can be swiped depends on the [property@Flap:transition-type]
  * value.
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 void
 adw_flap_set_swipe_to_open (AdwFlap  *self,
@@ -2492,7 +2502,7 @@ adw_flap_set_swipe_to_open (AdwFlap  *self,
  *
  * Returns: `TRUE` if @self can be closed with a swipe gesture
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 gboolean
 adw_flap_get_swipe_to_close (AdwFlap *self)
@@ -2512,7 +2522,7 @@ adw_flap_get_swipe_to_close (AdwFlap *self)
  * The area that can be swiped depends on the [property@Flap:transition-type]
  * value.
  *
- * Since: 1.0
+ * Deprecated: 1.4: See [the migration guide](migrating-to-breakpoints.html#replace-adwflap)
  */
 void
 adw_flap_set_swipe_to_close (AdwFlap  *self,
