@@ -3,7 +3,7 @@
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * Author: Alexander Mikhaylenko <alexander.mikhaylenko@puri.sm>
+ * Author: Alice Mikhaylenko <alice.mikhaylenko@puri.sm>
  */
 
 #include "config.h"
@@ -151,22 +151,6 @@
  * Since: 1.2
  */
 
-/**
- * AdwResponseAppearance:
- * @ADW_RESPONSE_DEFAULT: the default appearance.
- * @ADW_RESPONSE_SUGGESTED: used to denote important responses such as the
- *     affirmative action.
- * @ADW_RESPONSE_DESTRUCTIVE: used to draw attention to the potentially damaging
- *     consequences of using the response. This appearance acts as a warning to
- *     the user.
- *
- * Describes the possible styles of [class@MessageDialog] response buttons.
- *
- * See [method@MessageDialog.set_response_appearance].
- *
- * Since: 1.2
- */
-
 #define DIALOG_MARGIN 30
 #define DIALOG_MAX_WIDTH 550
 #define DIALOG_MIN_WIDTH 300
@@ -205,6 +189,8 @@ typedef struct
   GtkWindow *parent_window;
   int parent_width;
   int parent_height;
+
+  guint parent_state_idle_id;
 } AdwMessageDialogPrivate;
 
 static void adw_message_dialog_buildable_init (GtkBuildableIface *iface);
@@ -255,8 +241,8 @@ static void
 parent_size_cb (AdwMessageDialog *self)
 {
   AdwMessageDialogPrivate *priv = adw_message_dialog_get_instance_private (self);
-  int w = gtk_widget_get_allocated_width (GTK_WIDGET (priv->parent_window));
-  int h = gtk_widget_get_allocated_height (GTK_WIDGET (priv->parent_window));
+  int w = gtk_widget_get_width (GTK_WIDGET (priv->parent_window));
+  int h = gtk_widget_get_height (GTK_WIDGET (priv->parent_window));
 
   if (w == priv->parent_width && h == priv->parent_height)
       return;
@@ -266,6 +252,27 @@ parent_size_cb (AdwMessageDialog *self)
   gtk_widget_queue_resize (GTK_WIDGET (self));
 }
 
+static void
+parent_state_idle_cb (AdwMessageDialog *self)
+{
+  AdwMessageDialogPrivate *priv = adw_message_dialog_get_instance_private (self);
+
+  parent_size_cb (self);
+
+  priv->parent_state_idle_id = 0;
+}
+
+static void
+parent_state_cb (AdwMessageDialog *self)
+{
+  AdwMessageDialogPrivate *priv = adw_message_dialog_get_instance_private (self);
+
+  g_clear_handle_id (&priv->parent_state_idle_id, g_source_remove);
+
+  priv->parent_state_idle_id =
+    g_idle_add_once ((GSourceOnceFunc) parent_state_idle_cb, self);
+}
+
 static void
 parent_realize_cb (AdwMessageDialog *self)
 {
@@ -282,6 +289,8 @@ parent_realize_cb (AdwMessageDialog *self)
                             G_CALLBACK (parent_size_cb), self);
   g_signal_connect_swapped (surface, "notify::height",
                             G_CALLBACK (parent_size_cb), self);
+  g_signal_connect_swapped (surface, "notify::state",
+                            G_CALLBACK (parent_state_cb), self);
 
   parent_size_cb (self);
 }
@@ -299,6 +308,9 @@ parent_unrealize_cb (AdwMessageDialog *self)
   g_signal_handlers_disconnect_by_func (surface,
                                         G_CALLBACK (parent_size_cb),
                                         self);
+  g_signal_handlers_disconnect_by_func (surface,
+                                        G_CALLBACK (parent_state_cb),
+                                        self);
 
   priv->parent_width = -1;
   priv->parent_height = -1;
@@ -309,6 +321,8 @@ parent_window_notify_cb (AdwMessageDialog *self)
 {
   AdwMessageDialogPrivate *priv = adw_message_dialog_get_instance_private (self);
 
+  g_clear_handle_id (&priv->parent_state_idle_id, g_source_remove);
+
   priv->parent_window = NULL;
   priv->parent_width = -1;
   priv->parent_height = -1;
@@ -324,6 +338,8 @@ set_parent (AdwMessageDialog *self,
     return;
 
   if (priv->parent_window) {
+    g_clear_handle_id (&priv->parent_state_idle_id, g_source_remove);
+
     g_signal_handlers_disconnect_by_func (priv->parent_window,
                                           G_CALLBACK (parent_realize_cb),
                                           self);
@@ -387,6 +403,7 @@ create_response_button (AdwMessageDialog *self,
   GtkWidget *button = gtk_button_new_with_mnemonic (info->label);
 
   gtk_widget_add_css_class (button, "flat");
+  gtk_button_set_can_shrink (GTK_BUTTON (button), TRUE);
 
   switch (info->appearance) {
   case ADW_RESPONSE_SUGGESTED:
@@ -519,10 +536,10 @@ measure_responses_do (AdwMessageDialog *self,
 
     if (horiz == compact) {
       min = MAX (min, child_min);
-      nat = MAX (nat, child_min);
+      nat = MAX (nat, child_nat);
     } else if (horiz) {
       button_min = MAX (button_min, child_min);
-      button_nat = MAX (button_nat, child_min);
+      button_nat = MAX (button_nat, child_nat);
       n_buttons++;
     } else {
       min += child_min;
@@ -535,7 +552,7 @@ measure_responses_do (AdwMessageDialog *self,
 
     if (horiz == compact) {
         min = MAX (min, child_min);
-        nat = MAX (nat, child_min);
+        nat = MAX (nat, child_nat);
       } else {
         min += child_min;
         nat += child_nat;
@@ -575,12 +592,12 @@ measure_responses (GtkWidget      *widget,
     measure_responses_do (self, TRUE, orientation, minimum, NULL);
     measure_responses_do (self, FALSE, orientation, NULL, natural);
   } else {
-    int wide_min = 0;
+    int wide_nat = 0;
 
     if (for_size >= 0)
-      measure_responses_do (self, FALSE, GTK_ORIENTATION_HORIZONTAL, &wide_min, NULL);
+      measure_responses_do (self, FALSE, GTK_ORIENTATION_HORIZONTAL, NULL, &wide_nat);
 
-    measure_responses_do (self, for_size >= 0 && for_size < wide_min,
+    measure_responses_do (self, for_size >= 0 && for_size < wide_nat,
                           orientation, minimum, natural);
   }
 
@@ -599,11 +616,11 @@ allocate_responses (GtkWidget *widget,
   AdwMessageDialog *self = ADW_MESSAGE_DIALOG (gtk_widget_get_root (widget));
   AdwMessageDialogPrivate *priv = adw_message_dialog_get_instance_private (self);
   gboolean compact;
-  int wide_min;
+  int wide_nat;
 
-  measure_responses_do (self, FALSE, GTK_ORIENTATION_HORIZONTAL, &wide_min, NULL);
+  measure_responses_do (self, FALSE, GTK_ORIENTATION_HORIZONTAL, NULL, &wide_nat);
 
-  compact = wide_min > width;
+  compact = wide_nat > width;
 
   if (compact)
     gtk_widget_add_css_class (widget, "compact");
@@ -715,7 +732,7 @@ adw_message_dialog_measure (GtkWidget      *widget,
                                                                NULL, NULL);
 
   if (orientation == GTK_ORIENTATION_HORIZONTAL) {
-    int wide_min, narrow_nat;
+    int wide_nat, narrow_nat;
 
     min_size = MAX (min_size, DIALOG_MIN_WIDTH);
 
@@ -726,12 +743,12 @@ adw_message_dialog_measure (GtkWidget      *widget,
       max_size = DIALOG_MAX_WIDTH;
     }
 
-    measure_responses_do (self, FALSE, GTK_ORIENTATION_HORIZONTAL, &wide_min, NULL);
+    measure_responses_do (self, FALSE, GTK_ORIENTATION_HORIZONTAL, NULL, &wide_nat);
     measure_responses_do (self, TRUE, GTK_ORIENTATION_HORIZONTAL, NULL, &narrow_nat);
 
     narrow_nat = MAX (narrow_nat, DIALOG_MIN_WIDTH);
 
-    if (max_size < wide_min)
+    if (max_size < wide_nat)
       max_size = MIN (max_size, narrow_nat);
   } else {
     max_size = priv->parent_height - DIALOG_MARGIN * 2;
@@ -1369,12 +1386,9 @@ adw_message_dialog_set_heading (AdwMessageDialog *self,
 
   priv = adw_message_dialog_get_instance_private (self);
 
-  if (heading == priv->heading)
+  if (!g_set_str (&priv->heading, heading))
     return;
 
-  g_free (priv->heading);
-  priv->heading = g_strdup (heading);
-
   gtk_label_set_label (GTK_LABEL (priv->heading_label), heading);
   gtk_widget_set_visible (priv->heading_label, heading && *heading);
 
@@ -1580,12 +1594,9 @@ adw_message_dialog_set_body (AdwMessageDialog *self,
 
   priv = adw_message_dialog_get_instance_private (self);
 
-  if (body == priv->body)
+  if (!g_set_str (&priv->body, body))
     return;
 
-  g_free (priv->body);
-  priv->body = g_strdup (body);
-
   gtk_label_set_label (GTK_LABEL (priv->body_label), body);
   gtk_widget_set_visible (priv->body_label, body && *body);
 
@@ -1784,6 +1795,10 @@ adw_message_dialog_set_extra_child (AdwMessageDialog *self,
   AdwMessageDialogPrivate *priv;
 
   g_return_if_fail (ADW_IS_MESSAGE_DIALOG (self));
+  g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
+
+  if (child)
+    g_return_if_fail (gtk_widget_get_parent (child) == NULL);
 
   priv = adw_message_dialog_get_instance_private (self);
 
@@ -1924,6 +1939,53 @@ adw_message_dialog_add_responses (AdwMessageDialog *self,
   va_end (args);
 }
 
+/**
+ * adw_message_dialog_remove_response:
+ * @self: a message dialog
+ * @id: the response ID
+ *
+ * Removes a response from @self.
+ *
+ * Since: 1.5
+ */
+void
+adw_message_dialog_remove_response (AdwMessageDialog *self,
+                                    const char       *id)
+{
+  AdwMessageDialogPrivate *priv;
+  ResponseInfo *info;
+
+  g_return_if_fail (ADW_IS_MESSAGE_DIALOG (self));
+  g_return_if_fail (id != NULL);
+
+  priv = adw_message_dialog_get_instance_private (self);
+  info = find_response (self, id);
+
+  if (!info) {
+    g_critical ("Trying to remove a response with id '%s' from an "
+                "AdwMessageDialog, but such a response does not exist",
+                id);
+    return;
+  }
+
+  if (priv->default_response == info->id)
+    gtk_window_set_default_widget (GTK_WINDOW (self), NULL);
+
+  gtk_widget_unparent (info->button);
+
+  if (info == priv->responses->data && priv->responses->next) {
+    ResponseInfo *next_info = priv->responses->next->data;
+    g_clear_pointer (&next_info->separator, gtk_widget_unparent);
+  } else {
+    g_clear_pointer (&info->separator, gtk_widget_unparent);
+  }
+
+  priv->responses = g_list_remove (priv->responses, info);
+  g_hash_table_remove (priv->id_to_response, id);
+
+  response_info_free (info);
+}
+
 /**
  * adw_message_dialog_get_response_label:
  * @self: a message dialog
@@ -1979,8 +2041,7 @@ adw_message_dialog_set_response_label (AdwMessageDialog *self,
 
   info = find_response (self, response);
 
-  g_free (info->label);
-  info->label = g_strdup (label);
+  g_set_str (&info->label, label);
 
   gtk_button_set_label (GTK_BUTTON (info->button), label);
 }
@@ -2338,7 +2399,7 @@ choose_cancelled_cb (GCancellable *cancellable,
 
 /**
  * adw_message_dialog_choose:
- * @self: (transfer full): a message dialog
+ * @self: a message dialog
  * @cancellable: (nullable): a `GCancellable` to cancel the operation
  * @callback: (scope async): a callback to call when the operation is complete
  * @user_data: (closure callback): data to pass to @callback
