@@ -8,7 +8,6 @@
 
 #include "adw-preferences-group-private.h"
 
-#include "adw-macros-private.h"
 #include "adw-preferences-row.h"
 #include "adw-widget-utils-private.h"
 
@@ -26,7 +25,7 @@
  * which in turn are represented by [class@PreferencesRow].
  *
  * To summarize the role of the preferences it gathers, a group can have both a
- * title and a description. The title will be used by [class@PreferencesWindow]
+ * title and a description. The title will be used by [class@PreferencesDialog]
  * to let the user look for a preference.
  *
  * ## AdwPreferencesGroup as GtkBuildable
@@ -46,8 +45,6 @@
  * ## Accessibility
  *
  * `AdwPreferencesGroup` uses the `GTK_ACCESSIBLE_ROLE_GROUP` role.
- *
- * Since: 1.0
  */
 
 typedef struct
@@ -74,8 +71,8 @@ static GtkBuildableIface *parent_buildable_iface;
 
 enum {
   PROP_0,
-  PROP_DESCRIPTION,
   PROP_TITLE,
+  PROP_DESCRIPTION,
   PROP_HEADER_SUFFIX,
   LAST_PROP,
 };
@@ -165,6 +162,22 @@ listbox_keynav_failed_cb (AdwPreferencesGroup *self,
                                  GTK_DIR_TAB_BACKWARD : GTK_DIR_TAB_FORWARD);
 }
 
+static gboolean
+row_has_title (AdwPreferencesRow *row,
+               gpointer           user_data)
+{
+  const char *title;
+
+  g_assert (ADW_IS_PREFERENCES_ROW (row));
+
+  if (!gtk_widget_get_visible (GTK_WIDGET (row)))
+    return FALSE;
+
+  title = adw_preferences_row_get_title (row);
+
+  return title && *title;
+}
+
 static void
 adw_preferences_group_get_property (GObject    *object,
                                     guint       prop_id,
@@ -174,12 +187,12 @@ adw_preferences_group_get_property (GObject    *object,
   AdwPreferencesGroup *self = ADW_PREFERENCES_GROUP (object);
 
   switch (prop_id) {
-  case PROP_DESCRIPTION:
-    g_value_set_string (value, adw_preferences_group_get_description (self));
-    break;
   case PROP_TITLE:
     g_value_set_string (value, adw_preferences_group_get_title (self));
     break;
+  case PROP_DESCRIPTION:
+    g_value_set_string (value, adw_preferences_group_get_description (self));
+    break;
   case PROP_HEADER_SUFFIX:
     g_value_set_object (value, adw_preferences_group_get_header_suffix (self));
     break;
@@ -197,12 +210,12 @@ adw_preferences_group_set_property (GObject      *object,
   AdwPreferencesGroup *self = ADW_PREFERENCES_GROUP (object);
 
   switch (prop_id) {
-  case PROP_DESCRIPTION:
-    adw_preferences_group_set_description (self, g_value_get_string (value));
-    break;
   case PROP_TITLE:
     adw_preferences_group_set_title (self, g_value_get_string (value));
     break;
+  case PROP_DESCRIPTION:
+    adw_preferences_group_set_description (self, g_value_get_string (value));
+    break;
   case PROP_HEADER_SUFFIX:
     adw_preferences_group_set_header_suffix (self, g_value_get_object (value));
     break;
@@ -217,9 +230,10 @@ adw_preferences_group_dispose (GObject *object)
   AdwPreferencesGroup *self = ADW_PREFERENCES_GROUP (object);
   AdwPreferencesGroupPrivate *priv = adw_preferences_group_get_instance_private (self);
 
-  gtk_widget_unparent (priv->box);
   g_clear_object (&priv->rows);
 
+  gtk_widget_dispose_template (GTK_WIDGET (self), ADW_TYPE_PREFERENCES_GROUP);
+
   G_OBJECT_CLASS (adw_preferences_group_parent_class)->dispose (object);
 }
 
@@ -235,48 +249,42 @@ adw_preferences_group_class_init (AdwPreferencesGroupClass *klass)
 
   widget_class->compute_expand = adw_widget_compute_expand;
 
-  /**
-   * AdwPreferencesGroup:description: (attributes org.gtk.Property.get=adw_preferences_group_get_description org.gtk.Property.set=adw_preferences_group_set_description)
-   *
-   * The description for this group of preferences.
-   *
-   * Since: 1.0
-   */
-  props[PROP_DESCRIPTION] =
-    g_param_spec_string ("description",
-                         "Description",
-                         "The description for this group of preferences",
-                         "",
-                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
-
   /**
    * AdwPreferencesGroup:title: (attributes org.gtk.Property.get=adw_preferences_group_get_title org.gtk.Property.set=adw_preferences_group_set_title)
    *
    * The title for this group of preferences.
-   *
-   * Since: 1.0
    */
   props[PROP_TITLE] =
-    g_param_spec_string ("title",
-                         "Title",
-                         "The title for this group of preferences",
+    g_param_spec_string ("title", NULL, NULL,
                          "",
-                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
+  /**
+   * AdwPreferencesGroup:description: (attributes org.gtk.Property.get=adw_preferences_group_get_description org.gtk.Property.set=adw_preferences_group_set_description)
+   *
+   * The description for this group of preferences.
+   */
+  props[PROP_DESCRIPTION] =
+    g_param_spec_string ("description", NULL, NULL,
+                         "",
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
   /**
    * AdwPreferencesGroup:header-suffix: (attributes org.gtk.Property.get=adw_preferences_group_get_header_suffix org.gtk.Property.set=adw_preferences_group_set_header_suffix)
    *
    * The header suffix widget.
    *
-   * Displayed above the list, next to the title and description. Suffixes are commonly used to show a button or a spinner for the whole group.
+   * Displayed above the list, next to the title and description.
+   *
+   * Suffixes are commonly used to show a button or a spinner for the whole
+   * group.
    *
    * Since: 1.1
    */
   props[PROP_HEADER_SUFFIX] =
-    g_param_spec_object ("header-suffix",
-                         "Header Suffix",
-                         "The suffix for this group's header",
+    g_param_spec_object ("header-suffix", NULL, NULL,
                          GTK_TYPE_WIDGET,
-                         G_PARAM_READWRITE);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
@@ -344,8 +352,6 @@ adw_preferences_group_buildable_init (GtkBuildableIface *iface)
  * Creates a new `AdwPreferencesGroup`.
  *
  * Returns: the newly created `AdwPreferencesGroup`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_preferences_group_new (void)
@@ -353,6 +359,60 @@ adw_preferences_group_new (void)
   return g_object_new (ADW_TYPE_PREFERENCES_GROUP, NULL);
 }
 
+/**
+ * adw_preferences_group_add:
+ * @self: a preferences group
+ * @child: the widget to add
+ *
+ * Adds a child to @self.
+ */
+void
+adw_preferences_group_add (AdwPreferencesGroup *self,
+                           GtkWidget           *child)
+{
+  AdwPreferencesGroupPrivate *priv;
+
+  g_return_if_fail (ADW_IS_PREFERENCES_GROUP (self));
+  g_return_if_fail (GTK_IS_WIDGET (child));
+  g_return_if_fail (gtk_widget_get_parent (child) == NULL);
+
+  priv = adw_preferences_group_get_instance_private (self);
+
+  if (ADW_IS_PREFERENCES_ROW (child))
+    gtk_list_box_append (priv->listbox, child);
+  else
+    gtk_box_append (priv->listbox_box, child);
+}
+
+/**
+ * adw_preferences_group_remove:
+ * @self: a preferences group
+ * @child: the child to remove
+ *
+ * Removes a child from @self.
+ */
+void
+adw_preferences_group_remove (AdwPreferencesGroup *self,
+                              GtkWidget           *child)
+{
+  AdwPreferencesGroupPrivate *priv;
+  GtkWidget *parent;
+
+  g_return_if_fail (ADW_IS_PREFERENCES_GROUP (self));
+  g_return_if_fail (GTK_IS_WIDGET (child));
+
+  priv = adw_preferences_group_get_instance_private (self);
+
+  parent = gtk_widget_get_parent (child);
+
+  if (parent == GTK_WIDGET (priv->listbox))
+    gtk_list_box_remove (priv->listbox, child);
+  else if (parent == GTK_WIDGET (priv->listbox_box))
+    gtk_box_remove (priv->listbox_box, child);
+  else
+    ADW_CRITICAL_CANNOT_REMOVE_CHILD (self, child);
+}
+
 /**
  * adw_preferences_group_get_title: (attributes org.gtk.Method.get_property=title)
  * @self: a preferences group
@@ -360,8 +420,6 @@ adw_preferences_group_new (void)
  * Gets the title of @self.
  *
  * Returns: the title of @self
- *
- * Since: 1.0
  */
 const char *
 adw_preferences_group_get_title (AdwPreferencesGroup *self)
@@ -381,8 +439,6 @@ adw_preferences_group_get_title (AdwPreferencesGroup *self)
  * @title: the title
  *
  * Sets the title for @self.
- *
- * Since: 1.0
  */
 void
 adw_preferences_group_set_title (AdwPreferencesGroup *self,
@@ -411,8 +467,6 @@ adw_preferences_group_set_title (AdwPreferencesGroup *self,
  * Gets the description of @self.
  *
  * Returns: (nullable): the description of @self
- *
- * Since: 1.0
  */
 const char *
 adw_preferences_group_get_description (AdwPreferencesGroup *self)
@@ -432,8 +486,6 @@ adw_preferences_group_get_description (AdwPreferencesGroup *self)
  * @description: (nullable): the description
  *
  * Sets the description for @self.
- *
- * Since: 1.0
  */
 void
 adw_preferences_group_set_description (AdwPreferencesGroup *self,
@@ -455,109 +507,6 @@ adw_preferences_group_set_description (AdwPreferencesGroup *self,
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_DESCRIPTION]);
 }
 
-static gboolean
-row_has_title (AdwPreferencesRow *row,
-               gpointer           user_data)
-{
-  const char *title;
-
-  g_assert (ADW_IS_PREFERENCES_ROW (row));
-
-  if (!gtk_widget_get_visible (GTK_WIDGET (row)))
-    return FALSE;
-
-  title = adw_preferences_row_get_title (row);
-
-  return title && *title;
-}
-
-/**
- * adw_preferences_group_get_rows:
- * @self: a preferences group
- *
- * Gets a [iface@Gio.ListModel] that contains the rows of the group.
- *
- * This can be used to keep an up-to-date view.
- *
- * Returns: (transfer full): a list model for the group's rows
- *
- * Since: 1.0
- */
-GListModel *
-adw_preferences_group_get_rows (AdwPreferencesGroup *self)
-{
-  AdwPreferencesGroupPrivate *priv;
-  GtkCustomFilter *filter;
-  GListModel *model;
-
-  g_return_val_if_fail (ADW_IS_PREFERENCES_GROUP (self), NULL);
-
-  priv = adw_preferences_group_get_instance_private (self);
-
-  filter = gtk_custom_filter_new ((GtkCustomFilterFunc) row_has_title, NULL, NULL);
-  model = gtk_widget_observe_children (GTK_WIDGET (priv->listbox));
-  model = G_LIST_MODEL (gtk_filter_list_model_new (model, GTK_FILTER (filter)));
-
-  return model;
-}
-
-/**
- * adw_preferences_group_add:
- * @self: a preferences group
- * @child: the widget to add
- *
- * Adds a child to @self.
- *
- * Since: 1.0
- */
-void
-adw_preferences_group_add (AdwPreferencesGroup *self,
-                           GtkWidget           *child)
-{
-  AdwPreferencesGroupPrivate *priv;
-
-  g_return_if_fail (ADW_IS_PREFERENCES_GROUP (self));
-  g_return_if_fail (GTK_IS_WIDGET (child));
-
-  priv = adw_preferences_group_get_instance_private (self);
-
-  if (ADW_IS_PREFERENCES_ROW (child))
-    gtk_list_box_append (priv->listbox, child);
-  else
-    gtk_box_append (priv->listbox_box, child);
-}
-
-/**
- * adw_preferences_group_remove:
- * @self: a preferences group
- * @child: the child to remove
- *
- * Removes a child from @self.
- *
- * Since: 1.0
- */
-void
-adw_preferences_group_remove (AdwPreferencesGroup *self,
-                              GtkWidget           *child)
-{
-  AdwPreferencesGroupPrivate *priv;
-  GtkWidget *parent;
-
-  g_return_if_fail (ADW_IS_PREFERENCES_GROUP (self));
-  g_return_if_fail (GTK_IS_WIDGET (child));
-
-  priv = adw_preferences_group_get_instance_private (self);
-
-  parent = gtk_widget_get_parent (child);
-
-  if (parent == GTK_WIDGET (priv->listbox))
-    gtk_list_box_remove (priv->listbox, child);
-  else if (parent == GTK_WIDGET (priv->listbox_box))
-    gtk_box_remove (priv->listbox_box, child);
-  else
-    ADW_CRITICAL_CANNOT_REMOVE_CHILD (self, child);
-}
-
 /**
  * adw_preferences_group_get_header_suffix:
  * @self: a `AdwPreferencesGroup`
@@ -587,6 +536,10 @@ adw_preferences_group_get_header_suffix (AdwPreferencesGroup *self)
  *
  * Sets the suffix for @self's header.
  *
+ * Displayed above the list, next to the title and description.
+ *
+ * Suffixes are commonly used to show a button or a spinner for the whole group.
+ *
  * Since: 1.1
  */
 void
@@ -598,6 +551,9 @@ adw_preferences_group_set_header_suffix (AdwPreferencesGroup *self,
   g_return_if_fail (ADW_IS_PREFERENCES_GROUP (self));
   g_return_if_fail (suffix == NULL || GTK_IS_WIDGET (suffix));
 
+  if (suffix)
+    g_return_if_fail (gtk_widget_get_parent (suffix) == NULL);
+
   priv = adw_preferences_group_get_instance_private (self);
 
   if (suffix == priv->header_suffix)
@@ -613,3 +569,31 @@ adw_preferences_group_set_header_suffix (AdwPreferencesGroup *self,
 
   update_header_visibility (self);
 }
+
+/**
+ * adw_preferences_group_get_rows:
+ * @self: a preferences group
+ *
+ * Gets a [iface@Gio.ListModel] that contains the rows of the group.
+ *
+ * This can be used to keep an up-to-date view.
+ *
+ * Returns: (transfer full): a list model for the group's rows
+ */
+GListModel *
+adw_preferences_group_get_rows (AdwPreferencesGroup *self)
+{
+  AdwPreferencesGroupPrivate *priv;
+  GtkCustomFilter *filter;
+  GListModel *model;
+
+  g_return_val_if_fail (ADW_IS_PREFERENCES_GROUP (self), NULL);
+
+  priv = adw_preferences_group_get_instance_private (self);
+
+  filter = gtk_custom_filter_new ((GtkCustomFilterFunc) row_has_title, NULL, NULL);
+  model = gtk_widget_observe_children (GTK_WIDGET (priv->listbox));
+  model = G_LIST_MODEL (gtk_filter_list_model_new (model, GTK_FILTER (filter)));
+
+  return model;
+}
