@@ -13,15 +13,13 @@
  *
  * A [class@Gtk.ListBoxRow] used to present preferences.
  *
- * The `AdwPreferencesRow` widget has a title that [class@PreferencesWindow]
+ * The `AdwPreferencesRow` widget has a title that [class@PreferencesDialog]
  * will use to let the user look for a preference. It doesn't present the title
  * in any way and lets you present the preference as you please.
  *
  * [class@ActionRow] and its derivatives are convenient to use as preference
  * rows as they take care of presenting the preference's title while letting you
  * compose the inputs of the preference around it.
- *
- * Since: 1.0
  */
 
 typedef struct
@@ -30,6 +28,7 @@ typedef struct
 
   gboolean use_underline;
   gboolean title_selectable;
+  gboolean use_markup;
 } AdwPreferencesRowPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (AdwPreferencesRow, adw_preferences_row, GTK_TYPE_LIST_BOX_ROW)
@@ -39,6 +38,7 @@ enum {
   PROP_TITLE,
   PROP_USE_UNDERLINE,
   PROP_TITLE_SELECTABLE,
+  PROP_USE_MARKUP,
   LAST_PROP,
 };
 
@@ -51,7 +51,6 @@ adw_preferences_row_get_property (GObject    *object,
                                   GParamSpec *pspec)
 {
   AdwPreferencesRow *self = ADW_PREFERENCES_ROW (object);
-
   switch (prop_id) {
   case PROP_TITLE:
     g_value_set_string (value, adw_preferences_row_get_title (self));
@@ -62,6 +61,9 @@ adw_preferences_row_get_property (GObject    *object,
   case PROP_TITLE_SELECTABLE:
     g_value_set_boolean (value, adw_preferences_row_get_title_selectable (self));
     break;
+  case PROP_USE_MARKUP:
+    g_value_set_boolean (value, adw_preferences_row_get_use_markup (self));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -85,6 +87,9 @@ adw_preferences_row_set_property (GObject      *object,
   case PROP_TITLE_SELECTABLE:
     adw_preferences_row_set_title_selectable (self, g_value_get_boolean (value));
     break;
+  case PROP_USE_MARKUP:
+    adw_preferences_row_set_use_markup (self, g_value_get_boolean (value));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -115,12 +120,11 @@ adw_preferences_row_class_init (AdwPreferencesRowClass *klass)
    *
    * The title of the preference represented by this row.
    *
-   * Since: 1.0
+   * The title is interpreted as Pango markup unless
+   * [property@PreferencesRow:use-markup] is set to `FALSE`.
    */
   props[PROP_TITLE] =
-    g_param_spec_string ("title",
-                         "Title",
-                         "The title of the preference represented by this row",
+    g_param_spec_string ("title", NULL, NULL,
                          "",
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
@@ -128,15 +132,11 @@ adw_preferences_row_class_init (AdwPreferencesRowClass *klass)
    * AdwPreferencesRow:use-underline: (attributes org.gtk.Property.get=adw_preferences_row_get_use_underline org.gtk.Property.set=adw_preferences_row_set_use_underline)
    *
    * Whether an embedded underline in the title indicates a mnemonic.
-   *
-   * Since: 1.0
    */
   props[PROP_USE_UNDERLINE] =
-    g_param_spec_boolean ("use-underline",
-                          "Use underline",
-                          "Whether an embedded underline in the title indicates a mnemonic",
+    g_param_spec_boolean ("use-underline", NULL, NULL,
                           FALSE,
-                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwPreferencesRow:title-selectable: (attributes org.gtk.Property.get=adw_preferences_row_get_title_selectable org.gtk.Property.set=adw_preferences_row_set_title_selectable)
@@ -148,12 +148,26 @@ adw_preferences_row_class_init (AdwPreferencesRowClass *klass)
    * Since: 1.1
    */
   props[PROP_TITLE_SELECTABLE] =
-    g_param_spec_boolean ("title-selectable",
-                          "Title selectable",
-                          "Whether the title should be selectable (i.e. the user can copy it)",
+    g_param_spec_boolean ("title-selectable", NULL, NULL,
                           FALSE,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
+  /**
+   * AdwPreferencesRow:use-markup: (attributes org.gtk.Property.get=adw_preferences_row_get_use_markup org.gtk.Property.set=adw_preferences_row_set_use_markup)
+   *
+   * Whether to use Pango markup for the title label.
+   *
+   * Subclasses may also use it for other labels, such as subtitle.
+   *
+   * See also [func@Pango.parse_markup].
+   *
+   * Since: 1.2
+   */
+  props[PROP_USE_MARKUP] =
+    g_param_spec_boolean ("use-markup", NULL, NULL,
+                          TRUE,
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
   g_object_class_install_properties (object_class, LAST_PROP, props);
 }
 
@@ -162,6 +176,7 @@ adw_preferences_row_init (AdwPreferencesRow *self)
 {
     AdwPreferencesRowPrivate *priv = adw_preferences_row_get_instance_private (self);
     priv->title = g_strdup ("");
+    priv->use_markup = TRUE;
 }
 
 /**
@@ -170,8 +185,6 @@ adw_preferences_row_init (AdwPreferencesRow *self)
  * Creates a new `AdwPreferencesRow`.
  *
  * Returns: the newly created `AdwPreferencesRow`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_preferences_row_new (void)
@@ -186,8 +199,6 @@ adw_preferences_row_new (void)
  * Gets the title of the preference represented by @self.
  *
  * Returns: the title
- *
- * Since: 1.0
  */
 const char *
 adw_preferences_row_get_title (AdwPreferencesRow *self)
@@ -208,7 +219,8 @@ adw_preferences_row_get_title (AdwPreferencesRow *self)
  *
  * Sets the title of the preference represented by @self.
  *
- * Since: 1.0
+ * The title is interpreted as Pango markup unless
+ * [property@PreferencesRow:use-markup] is set to `FALSE`.
  */
 void
 adw_preferences_row_set_title (AdwPreferencesRow *self,
@@ -220,12 +232,9 @@ adw_preferences_row_set_title (AdwPreferencesRow *self,
 
   priv = adw_preferences_row_get_instance_private (self);
 
-  if (g_strcmp0 (priv->title, title) == 0)
+  if (!g_set_str (&priv->title, title ? title : ""))
     return;
 
-  g_free (priv->title);
-  priv->title = g_strdup (title ? title : "");
-
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_TITLE]);
 }
 
@@ -236,8 +245,6 @@ adw_preferences_row_set_title (AdwPreferencesRow *self,
  * Gets whether an embedded underline in the title indicates a mnemonic.
  *
  * Returns: whether an embedded underline in the title indicates a mnemonic
- *
- * Since: 1.0
  */
 gboolean
 adw_preferences_row_get_use_underline (AdwPreferencesRow *self)
@@ -257,8 +264,6 @@ adw_preferences_row_get_use_underline (AdwPreferencesRow *self)
  * @use_underline: `TRUE` if underlines in the text indicate mnemonics
  *
  * Sets whether an embedded underline in the title indicates a mnemonic.
- *
- * Since: 1.0
  */
 void
 adw_preferences_row_set_use_underline (AdwPreferencesRow *self,
@@ -307,6 +312,8 @@ adw_preferences_row_get_title_selectable (AdwPreferencesRow *self)
  *
  * Sets whether the user can copy the title from the label
  *
+ * See also [property@Gtk.Label:selectable].
+ *
  * Since: 1.1
  */
 void
@@ -326,3 +333,58 @@ adw_preferences_row_set_title_selectable (AdwPreferencesRow *self,
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_TITLE_SELECTABLE]);
 }
+
+/**
+ * adw_preferences_row_get_use_markup: (attributes org.gtk.Method.get_property=use-markup)
+ * @self: a preferences row
+ *
+ * Gets whether to use Pango markup for the title label.
+ *
+ * Returns: whether to use markup
+ *
+ * Since: 1.2
+ */
+gboolean
+adw_preferences_row_get_use_markup (AdwPreferencesRow *self)
+{
+  AdwPreferencesRowPrivate *priv;
+
+  g_return_val_if_fail (ADW_IS_PREFERENCES_ROW (self), FALSE);
+
+  priv = adw_preferences_row_get_instance_private (self);
+
+  return priv->use_markup;
+}
+
+/**
+ * adw_preferences_row_set_use_markup: (attributes org.gtk.Method.set_property=use-markup)
+ * @self: a preferences row
+ * @use_markup: whether to use markup
+ *
+ * Sets whether to use Pango markup for the title label.
+ *
+ * Subclasses may also use it for other labels, such as subtitle.
+ *
+ * See also [func@Pango.parse_markup].
+ *
+ * Since: 1.2
+ */
+void
+adw_preferences_row_set_use_markup (AdwPreferencesRow *self,
+                                    gboolean           use_markup)
+{
+  AdwPreferencesRowPrivate *priv;
+
+  g_return_if_fail (ADW_IS_PREFERENCES_ROW (self));
+
+  priv = adw_preferences_row_get_instance_private (self);
+
+  use_markup = !!use_markup;
+
+  if (priv->use_markup == use_markup)
+    return;
+
+  priv->use_markup = use_markup;
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_USE_MARKUP]);
+}
