@@ -8,7 +8,6 @@
 
 #include "adw-status-page.h"
 
-#include "adw-macros-private.h"
 #include "adw-widget-utils-private.h"
 
 /**
@@ -31,8 +30,6 @@
  * `AdwStatusPage` can use the
  * [`.compact`](style-classes.html#compact-status-page) style class for when it
  * needs to fit into a small space such a sidebar or a popover.
- *
- * Since: 1.0
  */
 
 enum {
@@ -159,9 +156,10 @@ adw_status_page_dispose (GObject *object)
 
   adw_status_page_set_child (self, NULL);
 
-  gtk_widget_unparent (self->scrolled_window);
   self->user_widget = NULL;
 
+  gtk_widget_dispose_template (GTK_WIDGET (self), ADW_TYPE_STATUS_PAGE);
+
   G_OBJECT_CLASS (adw_status_page_parent_class)->dispose (object);
 }
 
@@ -195,8 +193,6 @@ adw_status_page_class_init (AdwStatusPageClass *klass)
    * The name of the icon to be used.
    *
    * Changing this will set [property@StatusPage:paintable] to `NULL`.
-   *
-   * Since: 1.0
    */
   props[PROP_ICON_NAME] =
     g_param_spec_string ("icon-name", NULL, NULL,
@@ -209,8 +205,6 @@ adw_status_page_class_init (AdwStatusPageClass *klass)
    * The paintable to be used.
    *
    * Changing this will set [property@StatusPage:icon-name] to `NULL`.
-   *
-   * Since: 1.0
    */
   props[PROP_PAINTABLE] =
     g_param_spec_object ("paintable", NULL, NULL,
@@ -221,8 +215,8 @@ adw_status_page_class_init (AdwStatusPageClass *klass)
    * AdwStatusPage:title: (attributes org.gtk.Property.get=adw_status_page_get_title org.gtk.Property.set=adw_status_page_set_title)
    *
    * The title to be displayed below the icon.
-   *
-   * Since: 1.0
+   * 
+   * It is not parsed as Pango markup.
    */
   props[PROP_TITLE] =
     g_param_spec_string ("title", NULL, NULL,
@@ -232,9 +226,7 @@ adw_status_page_class_init (AdwStatusPageClass *klass)
   /**
    * AdwStatusPage:description: (attributes org.gtk.Property.get=adw_status_page_get_description org.gtk.Property.set=adw_status_page_set_description)
    *
-   * The description to be displayed below the title.
-   *
-   * Since: 1.0
+   * The description markup to be displayed below the title.
    */
   props[PROP_DESCRIPTION] =
     g_param_spec_string ("description", NULL, NULL,
@@ -245,8 +237,6 @@ adw_status_page_class_init (AdwStatusPageClass *klass)
    * AdwStatusPage:child: (attributes org.gtk.Property.get=adw_status_page_get_child org.gtk.Property.set=adw_status_page_set_child)
    *
    * The child widget.
-   *
-   * Since: 1.0
    */
   props[PROP_CHILD] =
     g_param_spec_object ("child", NULL, NULL,
@@ -307,8 +297,6 @@ adw_status_page_buildable_init (GtkBuildableIface *iface)
  * Creates a new `AdwStatusPage`.
  *
  * Returns: the newly created `AdwStatusPage`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_status_page_new (void)
@@ -323,8 +311,6 @@ adw_status_page_new (void)
  * Gets the icon name for @self.
  *
  * Returns: (nullable): the icon name
- *
- * Since: 1.0
  */
 const char *
 adw_status_page_get_icon_name (AdwStatusPage *self)
@@ -342,8 +328,6 @@ adw_status_page_get_icon_name (AdwStatusPage *self)
  * Sets the icon name for @self.
  *
  * Changing this will set [property@StatusPage:paintable] to `NULL`.
- *
- * Since: 1.0
  */
 void
 adw_status_page_set_icon_name (AdwStatusPage *self,
@@ -361,8 +345,7 @@ adw_status_page_set_icon_name (AdwStatusPage *self,
     g_object_notify_by_pspec (G_OBJECT (self), props[PROP_PAINTABLE]);
   }
 
-  g_free (self->icon_name);
-  self->icon_name = g_strdup (icon_name);
+  g_set_str (&self->icon_name, icon_name);
   gtk_image_set_from_icon_name (self->image, self->icon_name);
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ICON_NAME]);
 
@@ -376,8 +359,6 @@ adw_status_page_set_icon_name (AdwStatusPage *self,
  * Gets the paintable for @self.
  *
  * Returns: (nullable) (transfer none): the paintable
- *
- * Since: 1.0
  */
 GdkPaintable *
 adw_status_page_get_paintable (AdwStatusPage *self)
@@ -395,8 +376,6 @@ adw_status_page_get_paintable (AdwStatusPage *self)
  * Sets the paintable for @self.
  *
  * Changing this will set [property@StatusPage:icon-name] to `NULL`.
- *
- * Since: 1.0
  */
 void
 adw_status_page_set_paintable (AdwStatusPage *self,
@@ -429,8 +408,6 @@ adw_status_page_set_paintable (AdwStatusPage *self,
  * Gets the title for @self.
  *
  * Returns: the title
- *
- * Since: 1.0
  */
 const char *
 adw_status_page_get_title (AdwStatusPage *self)
@@ -447,9 +424,7 @@ adw_status_page_get_title (AdwStatusPage *self)
  *
  * Sets the title for @self.
  *
- * The title is displayed below the icon.
- *
- * Since: 1.0
+ * The title is displayed below the icon. It is not parsed as Pango markup.
  */
 void
 adw_status_page_set_title (AdwStatusPage *self,
@@ -469,11 +444,9 @@ adw_status_page_set_title (AdwStatusPage *self,
  * adw_status_page_get_description: (attributes org.gtk.Method.get_property=description)
  * @self: a status page
  *
- * Gets the description for @self.
+ * Gets the description markup for @self.
  *
  * Returns: (nullable): the description
- *
- * Since: 1.0
  */
 const char *
 adw_status_page_get_description (AdwStatusPage *self)
@@ -488,11 +461,9 @@ adw_status_page_get_description (AdwStatusPage *self)
  * @self: a status page
  * @description: (nullable): the description
  *
- * Sets the description for @self.
+ * Sets the description markup for @self.
  *
- * The description is displayed below the title.
- *
- * Since: 1.0
+ * The description is displayed below the title. It is parsed as Pango markup.
  */
 void
 adw_status_page_set_description (AdwStatusPage *self,
@@ -515,8 +486,6 @@ adw_status_page_set_description (AdwStatusPage *self,
  * Gets the child widget of @self.
  *
  * Returns: (nullable) (transfer none): the child widget of @self
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_status_page_get_child (AdwStatusPage *self)
@@ -532,14 +501,16 @@ adw_status_page_get_child (AdwStatusPage *self)
  * @child: (nullable): the child widget
  *
  * Sets the child widget of @self.
- *
- * Since: 1.0
  */
 void
 adw_status_page_set_child (AdwStatusPage *self,
                            GtkWidget     *child)
 {
   g_return_if_fail (ADW_IS_STATUS_PAGE (self));
+  g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
+
+  if (child)
+    g_return_if_fail (gtk_widget_get_parent (child) == NULL);
 
   if (child == self->user_widget)
     return;
@@ -552,5 +523,5 @@ adw_status_page_set_child (AdwStatusPage *self,
   if (self->user_widget)
     gtk_box_append (self->toplevel_box, self->user_widget);
 
-  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_DESCRIPTION]);
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_CHILD]);
 }
