@@ -3,14 +3,14 @@
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * Author: Alexander Mikhaylenko <alexander.mikhaylenko@puri.sm>
+ * Author: Alice Mikhaylenko <alice.mikhaylenko@puri.sm>
  */
 
 #include "config.h"
 
 #include "adw-style-manager-private.h"
 
-#include "adw-macros-private.h"
+#include "adw-main-private.h"
 #include "adw-settings-private.h"
 #include <gtk/gtk.h>
 
@@ -29,8 +29,6 @@
  * @ADW_COLOR_SCHEME_FORCE_DARK: Always use dark appearance.
  *
  * Application color schemes for [property@StyleManager:color-scheme].
- *
- * Since: 1.0
  */
 
 /**
@@ -44,8 +42,6 @@
  * It allows to set the color scheme via the
  * [property@StyleManager:color-scheme] property, and to query the current
  * appearance, as well as whether a system-wide color scheme preference exists.
- *
- * Since: 1.0
  */
 
 struct _AdwStyleManager
@@ -121,15 +117,13 @@ register_display (GdkDisplayManager *display_manager,
                     NULL);
 }
 
-static gboolean
+static void
 enable_animations_cb (AdwStyleManager *self)
 {
   gtk_style_context_remove_provider_for_display (self->display,
                                                  GTK_STYLE_PROVIDER (self->animations_provider));
 
   self->animation_timeout_id = 0;
-
-  return G_SOURCE_REMOVE;
 }
 
 static void
@@ -176,9 +170,9 @@ update_stylesheet (AdwStyleManager *self)
   }
 
   self->animation_timeout_id =
-    g_timeout_add (SWITCH_DURATION,
-                   G_SOURCE_FUNC (enable_animations_cb),
-                   self);
+    g_timeout_add_once (SWITCH_DURATION,
+                        (GSourceOnceFunc) enable_animations_cb,
+                        self);
 }
 
 static gboolean
@@ -257,7 +251,7 @@ adw_style_manager_constructed (GObject *object)
                              self,
                              G_CONNECT_SWAPPED);
 
-    if (!g_getenv ("GTK_THEME")) {
+    if (!adw_is_granite_present () && !g_getenv ("GTK_THEME")) {
       g_object_set (gtk_settings_get_for_display (self->display),
                     "gtk-theme-name", "Adwaita-empty",
                     NULL);
@@ -274,9 +268,8 @@ adw_style_manager_constructed (GObject *object)
     }
 
     self->animations_provider = gtk_css_provider_new ();
-    gtk_css_provider_load_from_data (self->animations_provider,
-                                     "* { transition: none; }",
-                                     -1);
+    gtk_css_provider_load_from_string (self->animations_provider,
+                                       "* { transition: none; }");
   }
 
   self->settings = adw_settings_get_default ();
@@ -387,15 +380,11 @@ adw_style_manager_class_init (AdwStyleManagerClass *klass)
    *
    * The display will be `NULL` for the style manager returned by
    * [func@StyleManager.get_default].
-   *
-   * Since: 1.0
    */
   props[PROP_DISPLAY] =
-    g_param_spec_object ("display",
-                         "Display",
-                         "The display the style manager is associated with",
+    g_param_spec_object ("display", NULL, NULL,
                          GDK_TYPE_DISPLAY,
-                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
   /**
    * AdwStyleManager:color-scheme: (attributes org.gtk.Property.get=adw_style_manager_get_color_scheme org.gtk.Property.set=adw_style_manager_set_color_scheme)
@@ -431,16 +420,12 @@ adw_style_manager_class_init (AdwStyleManagerClass *klass)
    * The [property@StyleManager:system-supports-color-schemes] property can be
    * used to check if the current environment provides a color scheme
    * preference.
-   *
-   * Since: 1.0
    */
   props[PROP_COLOR_SCHEME] =
-    g_param_spec_enum ("color-scheme",
-                       "Color Scheme",
-                       "The current color scheme",
+    g_param_spec_enum ("color-scheme", NULL, NULL,
                        ADW_TYPE_COLOR_SCHEME,
                        ADW_COLOR_SCHEME_DEFAULT,
-                       G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwStyleManager:system-supports-color-schemes: (attributes org.gtk.Property.get=adw_style_manager_get_system_supports_color_schemes)
@@ -452,15 +437,11 @@ adw_style_manager_class_init (AdwStyleManagerClass *klass)
    * separate appearance switcher if it's set to `FALSE`.
    *
    * See [property@StyleManager:color-scheme].
-   *
-   * Since: 1.0
    */
   props[PROP_SYSTEM_SUPPORTS_COLOR_SCHEMES] =
-    g_param_spec_boolean ("system-supports-color-schemes",
-                          "System supports color schemes",
-                          "Whether the system supports color schemes",
+    g_param_spec_boolean ("system-supports-color-schemes", NULL, NULL,
                           FALSE,
-                          G_PARAM_READABLE);
+                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
    * AdwStyleManager:dark: (attributes org.gtk.Property.get=adw_style_manager_get_dark)
@@ -469,15 +450,11 @@ adw_style_manager_class_init (AdwStyleManagerClass *klass)
    *
    * This property can be used to query the current appearance, as requested via
    * [property@StyleManager:color-scheme].
-   *
-   * Since: 1.0
    */
   props[PROP_DARK] =
-    g_param_spec_boolean ("dark",
-                          "Dark",
-                          "Whether the application is using dark appearance",
+    g_param_spec_boolean ("dark", NULL, NULL,
                           FALSE,
-                          G_PARAM_READABLE);
+                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
    * AdwStyleManager:high-contrast: (attributes org.gtk.Property.get=adw_style_manager_get_high_contrast)
@@ -485,15 +462,11 @@ adw_style_manager_class_init (AdwStyleManagerClass *klass)
    * Whether the application is using high contrast appearance.
    *
    * This cannot be overridden by applications.
-   *
-   * Since: 1.0
    */
   props[PROP_HIGH_CONTRAST] =
-    g_param_spec_boolean ("high-contrast",
-                          "High Contrast",
-                          "Whether the application is using high contrast appearance",
+    g_param_spec_boolean ("high-contrast", NULL, NULL,
                           FALSE,
-                          G_PARAM_READABLE);
+                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   g_object_class_install_properties (object_class, LAST_PROP, props);
 }
@@ -544,8 +517,6 @@ adw_style_manager_ensure (void)
  * See [func@StyleManager.get_for_display].
  *
  * Returns: (transfer none): the default style manager
- *
- * Since: 1.0
  */
 AdwStyleManager *
 adw_style_manager_get_default (void)
@@ -568,8 +539,6 @@ adw_style_manager_get_default (void)
  * Most applications should use [func@StyleManager.get_default] instead.
  *
  * Returns: (transfer none): the style manager for @display
- *
- * Since: 1.0
  */
 AdwStyleManager *
 adw_style_manager_get_for_display (GdkDisplay *display)
@@ -593,9 +562,7 @@ adw_style_manager_get_for_display (GdkDisplay *display)
  * The display will be `NULL` for the style manager returned by
  * [func@StyleManager.get_default].
  *
- * Returns: (transfer none): (nullable): the display
- *
- * Since: 1.0
+ * Returns: (transfer none) (nullable): the display
  */
 GdkDisplay *
 adw_style_manager_get_display (AdwStyleManager *self)
@@ -612,8 +579,6 @@ adw_style_manager_get_display (AdwStyleManager *self)
  * Gets the requested application color scheme.
  *
  * Returns: the color scheme
- *
- * Since: 1.0
  */
 AdwColorScheme
 adw_style_manager_get_color_scheme (AdwStyleManager *self)
@@ -635,7 +600,30 @@ adw_style_manager_get_color_scheme (AdwStyleManager *self)
  * [property@StyleManager:dark] property can be used to query the current
  * effective appearance.
  *
- * Since: 1.0
+ * The `ADW_COLOR_SCHEME_PREFER_LIGHT` color scheme results in the application
+ * using light appearance unless the system prefers dark colors. This is the
+ * default value.
+ *
+ * The `ADW_COLOR_SCHEME_PREFER_DARK` color scheme results in the application
+ * using dark appearance, but can still switch to the light appearance if the
+ * system can prefers it, for example, when the high contrast preference is
+ * enabled.
+ *
+ * The `ADW_COLOR_SCHEME_FORCE_LIGHT` and `ADW_COLOR_SCHEME_FORCE_DARK` values
+ * ignore the system preference entirely. They are useful if the application
+ * wants to match its UI to its content or to provide a separate color scheme
+ * switcher.
+ *
+ * If a per-[class@Gdk.Display] style manager has its color scheme set to
+ * `ADW_COLOR_SCHEME_DEFAULT`, it will inherit the color scheme from the
+ * default style manager.
+ *
+ * For the default style manager, `ADW_COLOR_SCHEME_DEFAULT` is equivalent to
+ * `ADW_COLOR_SCHEME_PREFER_LIGHT`.
+ *
+ * The [property@StyleManager:system-supports-color-schemes] property can be
+ * used to check if the current environment provides a color scheme
+ * preference.
  */
 void
 adw_style_manager_set_color_scheme (AdwStyleManager *self,
@@ -674,9 +662,11 @@ adw_style_manager_set_color_scheme (AdwStyleManager *self,
  *
  * Gets whether the system supports color schemes.
  *
- * Returns: whether the system supports color schemes
+ * This can be used to check if the current environment provides a color scheme
+ * preference. For example, applications might want to show a separate
+ * appearance switcher if it's set to `FALSE`.
  *
- * Since: 1.0
+ * Returns: whether the system supports color schemes
  */
 gboolean
 adw_style_manager_get_system_supports_color_schemes (AdwStyleManager *self)
@@ -692,9 +682,10 @@ adw_style_manager_get_system_supports_color_schemes (AdwStyleManager *self)
  *
  * Gets whether the application is using dark appearance.
  *
- * Returns: whether the application is using dark appearance
+ * This can be used to query the current appearance, as requested via
+ * [property@StyleManager:color-scheme].
  *
- * Since: 1.0
+ * Returns: whether the application is using dark appearance
  */
 gboolean
 adw_style_manager_get_dark (AdwStyleManager *self)
@@ -710,9 +701,9 @@ adw_style_manager_get_dark (AdwStyleManager *self)
  *
  * Gets whether the application is using high contrast appearance.
  *
- * Returns: whether the application is using high contrast appearance
+ * This cannot be overridden by applications.
  *
- * Since: 1.0
+ * Returns: whether the application is using high contrast appearance
  */
 gboolean
 adw_style_manager_get_high_contrast (AdwStyleManager *self)
