@@ -3,7 +3,7 @@
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * Author: Alexander Mikhaylenko <alexander.mikhaylenko@puri.sm>
+ * Author: Alice Mikhaylenko <alice.mikhaylenko@puri.sm>
  */
 
 #include "config.h"
@@ -11,7 +11,6 @@
 #include "adw-tab-bar-private.h"
 
 #include "adw-bin.h"
-#include "adw-macros-private.h"
 #include "adw-tab-box-private.h"
 #include "adw-widget-utils-private.h"
 
@@ -26,7 +25,7 @@
  * </picture>
  *
  * The `AdwTabBar` widget is a tab bar that can be used with conjunction with
- * `AdwTabView`.
+ * `AdwTabView`. It is typically used as a top bar within [class@ToolbarView].
  *
  * `AdwTabBar` can autohide and can optionally contain action widgets on both
  * sides of the tabs.
@@ -38,8 +37,6 @@
  * ## CSS nodes
  *
  * `AdwTabBar` has a single CSS node with name `tabbar`.
- *
- * Since: 1.0
  */
 
 struct _AdwTabBar
@@ -59,6 +56,8 @@ struct _AdwTabBar
   AdwTabView *view;
   gboolean autohide;
 
+  GdkDragAction extra_drag_preferred_action;
+
   gboolean is_overflowing;
   gboolean resize_frozen;
 };
@@ -78,6 +77,8 @@ enum {
   PROP_EXPAND_TABS,
   PROP_INVERTED,
   PROP_IS_OVERFLOWING,
+  PROP_EXTRA_DRAG_PRELOAD,
+  PROP_EXTRA_DRAG_PREFERRED_ACTION,
   LAST_PROP
 };
 
@@ -85,6 +86,7 @@ static GParamSpec *props[LAST_PROP];
 
 enum {
   SIGNAL_EXTRA_DRAG_DROP,
+  SIGNAL_EXTRA_DRAG_VALUE,
   SIGNAL_LAST_SIGNAL,
 };
 
@@ -240,18 +242,49 @@ stop_kinetic_scrolling_cb (GtkScrolledWindow *scrolled_window)
   gtk_scrolled_window_set_kinetic_scrolling (scrolled_window, TRUE);
 }
 
+static void
+set_extra_drag_preferred_action (AdwTabBar    *self,
+                                 GdkDragAction preferred_action)
+{
+  self->extra_drag_preferred_action = preferred_action;
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_EXTRA_DRAG_PREFERRED_ACTION]);
+}
+
 static gboolean
-extra_drag_drop_cb (AdwTabBar  *self,
-                    AdwTabPage *page,
-                    GValue     *value)
+extra_drag_drop_cb (AdwTabBar    *self,
+                    AdwTabPage   *page,
+                    GValue       *value,
+                    GdkDragAction preferred_action)
 {
   gboolean ret = GDK_EVENT_PROPAGATE;
 
+  set_extra_drag_preferred_action (self, preferred_action);
   g_signal_emit (self, signals[SIGNAL_EXTRA_DRAG_DROP], 0, page, value, &ret);
+  set_extra_drag_preferred_action (self, 0);
 
   return ret;
 }
 
+static GdkDragAction
+extra_drag_value_cb (AdwTabBar  *self,
+                     AdwTabPage *page,
+                     GValue     *value)
+{
+  GdkDragAction preferred_action;
+
+  g_signal_emit (self, signals[SIGNAL_EXTRA_DRAG_VALUE], 0, page, value, &preferred_action);
+
+  return preferred_action;
+}
+
+static GdkDragAction
+extra_drag_value_notify (AdwTabBar *self,
+                         GValue    *value)
+{
+  return GDK_ACTION_ALL;
+}
+
 static void
 view_destroy_cb (AdwTabBar *self)
 {
@@ -301,7 +334,7 @@ adw_tab_bar_dispose (GObject *object)
 
   adw_tab_bar_set_view (self, NULL);
 
-  gtk_widget_unparent (GTK_WIDGET (self->revealer));
+  gtk_widget_dispose_template (GTK_WIDGET (self), ADW_TYPE_TAB_BAR);
 
   G_OBJECT_CLASS (adw_tab_bar_parent_class)->dispose (object);
 }
@@ -347,6 +380,14 @@ adw_tab_bar_get_property (GObject    *object,
     g_value_set_boolean (value, adw_tab_bar_get_is_overflowing (self));
     break;
 
+  case PROP_EXTRA_DRAG_PREFERRED_ACTION:
+    g_value_set_flags (value, adw_tab_bar_get_extra_drag_preferred_action (self));
+    break;
+
+  case PROP_EXTRA_DRAG_PRELOAD:
+    g_value_set_boolean (value, adw_tab_bar_get_extra_drag_preload (self));
+    break;
+
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -385,6 +426,10 @@ adw_tab_bar_set_property (GObject      *object,
     adw_tab_bar_set_inverted (self, g_value_get_boolean (value));
     break;
 
+  case PROP_EXTRA_DRAG_PRELOAD:
+    adw_tab_bar_set_extra_drag_preload (self, g_value_get_boolean (value));
+    break;
+
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -407,8 +452,6 @@ adw_tab_bar_class_init (AdwTabBarClass *klass)
    * AdwTabBar:view: (attributes org.gtk.Property.get=adw_tab_bar_get_view org.gtk.Property.set=adw_tab_bar_set_view)
    *
    * The tab view the tab bar controls.
-   *
-   * Since: 1.0
    */
   props[PROP_VIEW] =
     g_param_spec_object ("view", NULL, NULL,
@@ -419,8 +462,6 @@ adw_tab_bar_class_init (AdwTabBarClass *klass)
    * AdwTabBar:start-action-widget: (attributes org.gtk.Property.get=adw_tab_bar_get_start_action_widget org.gtk.Property.set=adw_tab_bar_set_start_action_widget)
    *
    * The widget shown before the tabs.
-   *
-   * Since: 1.0
    */
   props[PROP_START_ACTION_WIDGET] =
     g_param_spec_object ("start-action-widget", NULL, NULL,
@@ -431,8 +472,6 @@ adw_tab_bar_class_init (AdwTabBarClass *klass)
    * AdwTabBar:end-action-widget: (attributes org.gtk.Property.get=adw_tab_bar_get_end_action_widget org.gtk.Property.set=adw_tab_bar_set_end_action_widget)
    *
    * The widget shown after the tabs.
-   *
-   * Since: 1.0
    */
   props[PROP_END_ACTION_WIDGET] =
     g_param_spec_object ("end-action-widget", NULL, NULL,
@@ -448,8 +487,6 @@ adw_tab_bar_class_init (AdwTabBarClass *klass)
    * or 1 tab, no pinned tabs, and no tab is being transferred.
    *
    * See [property@TabBar:tabs-revealed].
-   *
-   * Since: 1.0
    */
   props[PROP_AUTOHIDE] =
     g_param_spec_boolean ("autohide", NULL, NULL,
@@ -462,8 +499,6 @@ adw_tab_bar_class_init (AdwTabBarClass *klass)
    * Whether the tabs are currently revealed.
    *
    * See [property@TabBar:autohide].
-   *
-   * Since: 1.0
    */
   props[PROP_TABS_REVEALED] =
     g_param_spec_boolean ("tabs-revealed", NULL, NULL,
@@ -477,8 +512,6 @@ adw_tab_bar_class_init (AdwTabBarClass *klass)
    *
    * If set to `TRUE`, the tabs will always vary width filling the whole width
    * when possible, otherwise tabs will always have the minimum possible size.
-   *
-   * Since: 1.0
    */
   props[PROP_EXPAND_TABS] =
     g_param_spec_boolean ("expand-tabs", NULL, NULL,
@@ -492,8 +525,6 @@ adw_tab_bar_class_init (AdwTabBarClass *klass)
    *
    * If set to `TRUE`, non-pinned tabs will have the close button at the
    * beginning and the indicator at the end rather than the opposite.
-   *
-   * Since: 1.0
    */
   props[PROP_INVERTED] =
     g_param_spec_boolean ("inverted", NULL, NULL,
@@ -506,14 +537,43 @@ adw_tab_bar_class_init (AdwTabBarClass *klass)
    * Whether the tab bar is overflowing.
    *
    * If `TRUE`, all tabs cannot be displayed at once and require scrolling.
-   *
-   * Since: 1.0
    */
   props[PROP_IS_OVERFLOWING] =
     g_param_spec_boolean ("is-overflowing", NULL, NULL,
                           FALSE,
                           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
+  /**
+   * AdwTabBar:extra-drag-preferred-action: (attributes org.gtk.Property.get=adw_tab_bar_get_extra_drag_preferred_action)
+   *
+   * The unique action on the `current-drop` of the
+   * [signal@TabBar::extra-drag-drop].
+   *
+   * This property should only be used during a [signal@TabBar::extra-drag-drop]
+   * and is always a subset of what was originally passed to
+   * [method@TabBar.setup_extra_drop_target].
+   *
+   * Since: 1.4
+   */
+  props[PROP_EXTRA_DRAG_PREFERRED_ACTION] =
+    g_param_spec_flags ("extra-drag-preferred-action", NULL, NULL,
+                       GDK_TYPE_DRAG_ACTION, 0,
+                       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
+  /**
+   * AdwTabBar:extra-drag-preload: (attributes org.gtk.Property.get=adw_tab_bar_get_extra_drag_preload org.gtk.Property.set=adw_tab_bar_set_extra_drag_preload)
+   *
+   * Whether the drop data should be preloaded on hover.
+   *
+   * See [property@Gtk.DropTarget:preload].
+   *
+   * Since: 1.3
+   */
+  props[PROP_EXTRA_DRAG_PRELOAD] =
+    g_param_spec_boolean ("extra-drag-preload", NULL, NULL,
+                          FALSE,
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
   /**
@@ -530,20 +590,51 @@ adw_tab_bar_class_init (AdwTabBarClass *klass)
    * See [signal@Gtk.DropTarget::drop].
    *
    * Returns: whether the drop was accepted for @page
-   *
-   * Since: 1.0
    */
   signals[SIGNAL_EXTRA_DRAG_DROP] =
     g_signal_new ("extra-drag-drop",
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST,
                   0,
-                  g_signal_accumulator_first_wins, NULL, NULL,
+                  g_signal_accumulator_first_wins,
+                  NULL, NULL,
                   G_TYPE_BOOLEAN,
                   2,
                   ADW_TYPE_TAB_PAGE,
                   G_TYPE_VALUE);
 
+  /**
+   * AdwTabBar::extra-drag-value:
+   * @self: a tab bar
+   * @page: the page matching the tab the content was dropped onto
+   * @value: the `GValue` being dropped
+   *
+   * This signal is emitted when the dropped content is preloaded.
+   *
+   * In order for data to be preloaded, [property@TabBar:extra-drag-preload]
+   * must be set to `TRUE`.
+   *
+   * The content must be of one of the types set up via
+   * [method@TabBar.setup_extra_drop_target].
+   *
+   * See [property@Gtk.DropTarget:value].
+   *
+   * Returns: the preferred action for the drop on @page
+   *
+   * Since: 1.3
+   */
+  signals[SIGNAL_EXTRA_DRAG_VALUE] =
+    g_signal_new ("extra-drag-value",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  0,
+                  g_signal_accumulator_first_wins,
+                  NULL, NULL,
+                  GDK_TYPE_DRAG_ACTION,
+                  2,
+                  ADW_TYPE_TAB_PAGE,
+                  G_TYPE_VALUE);
+
   gtk_widget_class_set_template_from_resource (widget_class,
                                                "/org/gnome/Adwaita/ui/adw-tab-bar.ui");
   gtk_widget_class_bind_template_child (widget_class, AdwTabBar, revealer);
@@ -556,10 +647,14 @@ adw_tab_bar_class_init (AdwTabBarClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, notify_resize_frozen_cb);
   gtk_widget_class_bind_template_callback (widget_class, stop_kinetic_scrolling_cb);
   gtk_widget_class_bind_template_callback (widget_class, extra_drag_drop_cb);
+  gtk_widget_class_bind_template_callback (widget_class, extra_drag_value_cb);
 
   gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BIN_LAYOUT);
   gtk_widget_class_set_css_name (widget_class, "tabbar");
 
+  g_signal_override_class_handler ("extra-drag-value", G_TYPE_FROM_CLASS (klass),
+                                   G_CALLBACK (extra_drag_value_notify));
+
   g_type_ensure (ADW_TYPE_TAB_BOX);
 }
 
@@ -617,13 +712,22 @@ adw_tab_bar_tabs_have_visible_focus (AdwTabBar *self)
   g_return_val_if_fail (ADW_IS_TAB_BAR (self), FALSE);
 
   pinned_focus_child = gtk_widget_get_focus_child (GTK_WIDGET (self->pinned_box));
+
+  if (pinned_focus_child) {
+    GtkWidget *tab = gtk_widget_get_first_child (pinned_focus_child);
+
+    if (gtk_widget_has_visible_focus (tab))
+      return TRUE;
+  }
+
   scroll_focus_child = gtk_widget_get_focus_child (GTK_WIDGET (self->box));
 
-  if (pinned_focus_child && gtk_widget_has_visible_focus (pinned_focus_child))
-    return TRUE;
+  if (scroll_focus_child) {
+    GtkWidget *tab = gtk_widget_get_first_child (scroll_focus_child);
 
-  if (scroll_focus_child && gtk_widget_has_visible_focus (scroll_focus_child))
-    return TRUE;
+    if (gtk_widget_has_visible_focus (tab))
+      return TRUE;
+  }
 
   return FALSE;
 }
@@ -634,8 +738,6 @@ adw_tab_bar_tabs_have_visible_focus (AdwTabBar *self)
  * Creates a new `AdwTabBar`.
  *
  * Returns: the newly created `AdwTabBar`
- *
- * Since: 1.0
  */
 AdwTabBar *
 adw_tab_bar_new (void)
@@ -650,8 +752,6 @@ adw_tab_bar_new (void)
  * Gets the tab view @self controls.
  *
  * Returns: (transfer none) (nullable): the view @self controls
- *
- * Since: 1.0
  */
 AdwTabView *
 adw_tab_bar_get_view (AdwTabBar *self)
@@ -667,8 +767,6 @@ adw_tab_bar_get_view (AdwTabBar *self)
  * @view: (nullable): a tab view
  *
  * Sets the tab view @self controls.
- *
- * Since: 1.0
  */
 void
 adw_tab_bar_set_view (AdwTabBar  *self,
@@ -746,8 +844,6 @@ adw_tab_bar_set_view (AdwTabBar  *self,
  * Gets the widget shown before the tabs.
  *
  * Returns: (transfer none) (nullable): the widget shown before the tabs
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_tab_bar_get_start_action_widget (AdwTabBar *self)
@@ -763,8 +859,6 @@ adw_tab_bar_get_start_action_widget (AdwTabBar *self)
  * @widget: (transfer none) (nullable): the widget to show before the tabs
  *
  * Sets the widget to show before the tabs.
- *
- * Since: 1.0
  */
 void
 adw_tab_bar_set_start_action_widget (AdwTabBar *self,
@@ -793,8 +887,6 @@ adw_tab_bar_set_start_action_widget (AdwTabBar *self,
  * Gets the widget shown after the tabs.
  *
  * Returns: (transfer none) (nullable): the widget shown after the tabs
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_tab_bar_get_end_action_widget (AdwTabBar *self)
@@ -810,8 +902,6 @@ adw_tab_bar_get_end_action_widget (AdwTabBar *self)
  * @widget: (transfer none) (nullable): the widget to show after the tabs
  *
  * Sets the widget to show after the tabs.
- *
- * Since: 1.0
  */
 void
 adw_tab_bar_set_end_action_widget (AdwTabBar *self,
@@ -840,8 +930,6 @@ adw_tab_bar_set_end_action_widget (AdwTabBar *self,
  * Gets whether the tabs automatically hide.
  *
  * Returns: whether the tabs automatically hide
- *
- * Since: 1.0
  */
 gboolean
 adw_tab_bar_get_autohide (AdwTabBar *self)
@@ -862,8 +950,6 @@ adw_tab_bar_get_autohide (AdwTabBar *self)
  * or 1 tab, no pinned tabs, and no tab is being transferred.
  *
  * See [property@TabBar:tabs-revealed].
- *
- * Since: 1.0
  */
 void
 adw_tab_bar_set_autohide (AdwTabBar *self,
@@ -892,8 +978,6 @@ adw_tab_bar_set_autohide (AdwTabBar *self,
  * See [property@TabBar:autohide].
  *
  * Returns: whether the tabs are currently revealed
- *
- * Since: 1.0
  */
 gboolean
 adw_tab_bar_get_tabs_revealed (AdwTabBar *self)
@@ -910,8 +994,6 @@ adw_tab_bar_get_tabs_revealed (AdwTabBar *self)
  * Gets whether tabs expand to full width.
  *
  * Returns: whether tabs expand to full width.
- *
- * Since: 1.0
  */
 gboolean
 adw_tab_bar_get_expand_tabs (AdwTabBar *self)
@@ -930,8 +1012,6 @@ adw_tab_bar_get_expand_tabs (AdwTabBar *self)
  *
  * If set to `TRUE`, the tabs will always vary width filling the whole width
  * when possible, otherwise tabs will always have the minimum possible size.
- *
- * Since: 1.0
  */
 void
 adw_tab_bar_set_expand_tabs (AdwTabBar *self,
@@ -956,8 +1036,6 @@ adw_tab_bar_set_expand_tabs (AdwTabBar *self,
  * Gets whether tabs use inverted layout.
  *
  * Returns: whether tabs use inverted layout
- *
- * Since: 1.0
  */
 gboolean
 adw_tab_bar_get_inverted (AdwTabBar *self)
@@ -976,8 +1054,6 @@ adw_tab_bar_get_inverted (AdwTabBar *self)
  *
  * If set to `TRUE`, non-pinned tabs will have the close button at the beginning
  * and the indicator at the end rather than the opposite.
- *
- * Since: 1.0
  */
 void
 adw_tab_bar_set_inverted (AdwTabBar *self,
@@ -1014,8 +1090,6 @@ adw_tab_bar_set_inverted (AdwTabBar *self,
  * it will be automatically selected.
  *
  * The [signal@TabBar::extra-drag-drop] signal can be used to handle the drop.
- *
- * Since: 1.0
  */
 void
 adw_tab_bar_setup_extra_drop_target (AdwTabBar     *self,
@@ -1030,6 +1104,68 @@ adw_tab_bar_setup_extra_drop_target (AdwTabBar     *self,
   adw_tab_box_setup_extra_drop_target (self->pinned_box, actions, types, n_types);
 }
 
+/**
+ * adw_tab_bar_get_extra_drag_preferred_action:
+ * @self: a tab bar
+ *
+ * Gets the current action during a drop on the extra_drop_target.
+ *
+ * Returns: the drag action of the current drop.
+ *
+ * Since: 1.4
+ */
+GdkDragAction
+adw_tab_bar_get_extra_drag_preferred_action (AdwTabBar *self)
+{
+  g_return_val_if_fail (ADW_IS_TAB_BAR (self), 0);
+
+  return self->extra_drag_preferred_action;
+}
+
+/**
+ * adw_tab_bar_get_extra_drag_preload:
+ * @self: a tab bar
+ *
+ * Gets whether drop data should be preloaded on hover.
+ *
+ * Returns: whether drop data should be preloaded on hover
+ *
+ * Since: 1.3
+ */
+gboolean
+adw_tab_bar_get_extra_drag_preload (AdwTabBar *self)
+{
+  g_return_val_if_fail (ADW_IS_TAB_BAR (self), FALSE);
+
+  return adw_tab_box_get_extra_drag_preload (self->box);
+}
+
+/**
+ * adw_tab_bar_set_extra_drag_preload:
+ * @self: a tab bar
+ * @preload: whether to preload drop data
+ *
+ * Sets whether drop data should be preloaded on hover.
+ *
+ * See [property@Gtk.DropTarget:preload].
+ *
+ * Since: 1.3
+ */
+void
+adw_tab_bar_set_extra_drag_preload (AdwTabBar *self,
+                                    gboolean   preload)
+{
+  g_return_if_fail (ADW_IS_TAB_BAR (self));
+
+  if (adw_tab_bar_get_extra_drag_preload (self) == preload)
+    return;
+
+  adw_tab_box_set_extra_drag_preload (self->box, preload);
+  adw_tab_box_set_extra_drag_preload (self->pinned_box, preload);
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_EXTRA_DRAG_PRELOAD]);
+}
+
 /**
  * adw_tab_bar_get_is_overflowing: (attributes org.gtk.Method.get_property=is-overflowing)
  * @self: a tab bar
@@ -1039,8 +1175,6 @@ adw_tab_bar_setup_extra_drop_target (AdwTabBar     *self,
  * If `TRUE`, all tabs cannot be displayed at once and require scrolling.
  *
  * Returns: whether @self is overflowing
- *
- * Since: 1.0
  */
 gboolean
 adw_tab_bar_get_is_overflowing (AdwTabBar *self)
