@@ -3,7 +3,7 @@
  *
  * SPDX-License-Identifier: LGPL-2.1+
  *
- * Author: Alexander Mikhaylenko <alexander.mikhaylenko@puri.sm>
+ * Author: Alice Mikhaylenko <alice.mikhaylenko@puri.sm>
  */
 
 #include "config.h"
@@ -125,6 +125,8 @@ struct _AdwTabOverview
   GtkWidget *transition_picture;
   gboolean transition_pinned;
 
+  GdkDragAction extra_drag_preferred_action;
+
   GtkWidget *last_focus;
 };
 
@@ -147,6 +149,7 @@ enum {
   PROP_SECONDARY_MENU,
   PROP_SHOW_START_TITLE_BUTTONS,
   PROP_SHOW_END_TITLE_BUTTONS,
+  PROP_EXTRA_DRAG_PREFERRED_ACTION,
   PROP_EXTRA_DRAG_PRELOAD,
   LAST_PROP
 };
@@ -160,6 +163,12 @@ enum {
   SIGNAL_LAST_SIGNAL,
 };
 
+typedef enum {
+  ANIMATION_NONE,
+  ANIMATION_IN,
+  ANIMATION_OUT,
+} AnimationDirection;
+
 static guint signals[SIGNAL_LAST_SIGNAL];
 
 #define ADW_TYPE_TAB_OVERVIEW_SCROLLABLE (adw_tab_overview_scrollable_get_type ())
@@ -528,19 +537,18 @@ adw_tab_overview_scrollable_size_allocate (GtkWidget *widget,
   /* The value may have changed during gtk_adjustment_configure() */
   value = floor (gtk_adjustment_get_value (self->vadjustment));
 
-  if (G_APPROX_VALUE (value, 0, DBL_EPSILON))
-    gtk_widget_add_css_class (self->overview, "scrolled-to-top");
-  else
-    gtk_widget_remove_css_class (self->overview, "scrolled-to-top");
-
   adw_tab_grid_set_visible_range (ADW_TAB_GRID (self->pinned_grid),
                                   CLAMP (value - self->pinned_grid_pos, 0, pinned_height),
                                   CLAMP (value - self->pinned_grid_pos + height - new_button_height, 0, pinned_height),
-                                  height - new_button_height);
+                                  height - new_button_height,
+                                  0,
+                                  CLAMP (self->pinned_grid_pos + pinned_height - height + new_button_height - value, 0, new_button_height));
   adw_tab_grid_set_visible_range (ADW_TAB_GRID (self->grid),
                                   CLAMP (value - self->grid_pos, 0, grid_height),
                                   CLAMP (value - self->grid_pos + height - new_button_height, 0, grid_height),
-                                  height - new_button_height);
+                                  height - new_button_height,
+                                  0,
+                                  CLAMP (self->grid_pos + grid_height - height + new_button_height - value, 0, new_button_height));
 
   if (self->scroll_animation_done) {
     g_clear_pointer (&self->scroll_animation_grid, adw_tab_grid_reset_scrolled_tab);
@@ -725,14 +733,27 @@ adw_tab_overview_scrollable_init (AdwTabOverviewScrollable *self)
                             G_CALLBACK (scroll_animation_done_cb), self);
 }
 
+static void
+set_extra_drag_preferred_action (AdwTabOverview *self,
+                                 GdkDragAction   preferred_action)
+{
+  self->extra_drag_preferred_action = preferred_action;
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_EXTRA_DRAG_PREFERRED_ACTION]);
+}
+
+
 static gboolean
 extra_drag_drop_cb (AdwTabOverview *self,
                     AdwTabPage     *page,
-                    GValue         *value)
+                    GValue         *value,
+                    GdkDragAction   preferred_action)
 {
   gboolean ret = GDK_EVENT_PROPAGATE;
 
+  set_extra_drag_preferred_action (self, preferred_action);
   g_signal_emit (self, signals[SIGNAL_EXTRA_DRAG_DROP], 0, page, value, &ret);
+  set_extra_drag_preferred_action (self, 0);
 
   return ret;
 }
@@ -749,6 +770,13 @@ extra_drag_value_cb (AdwTabOverview *self,
   return preferred_action;
 }
 
+static GdkDragAction
+extra_drag_value_notify (AdwTabOverview *self,
+                         GValue         *value)
+{
+  return GDK_ACTION_ALL;
+}
+
 static void
 empty_changed_cb (AdwTabOverview *self)
 {
@@ -897,7 +925,7 @@ notify_n_pages_cb (AdwTabOverview *self)
   n_pages = adw_tab_view_get_n_pages (self->view);
 
   /* Translators: Tab overview title, %u is the number of open tabs */
-  title_str = g_strdup_printf (ngettext ("%u Tab", "%u Tabs", n_pages), n_pages);
+  title_str = g_strdup_printf (dngettext (GETTEXT_PACKAGE, "%u Tab", "%u Tabs", n_pages), n_pages);
 
   adw_window_title_set_title (ADW_WINDOW_TITLE (self->title), title_str);
 
@@ -956,15 +984,17 @@ open_animation_value_cb (double          value,
 }
 
 static void
-set_overview_visible (AdwTabOverview *self,
-                      gboolean        visible,
-                      gboolean        animating)
+set_overview_visible (AdwTabOverview     *self,
+                      gboolean            visible,
+                      AnimationDirection  direction)
 {
+  gboolean animating = direction != ANIMATION_NONE;
+
   gtk_widget_set_child_visible (self->overview, visible || animating);
   gtk_widget_set_can_target (self->overview, visible);
   gtk_widget_set_can_focus (self->overview, visible);
   gtk_widget_set_can_target (self->child_bin, !visible && !animating);
-  gtk_widget_set_can_focus (self->child_bin, !visible && !animating);
+  gtk_widget_set_can_focus (self->child_bin, !visible && direction != ANIMATION_IN);
 
   if (visible || animating)
     gtk_widget_add_css_class (self->child_bin, "background");
@@ -982,7 +1012,7 @@ open_animation_done_cb (AdwTabOverview *self)
     self->transition_thumbnail = NULL;
   }
 
-  set_overview_visible (self, self->is_open, FALSE);
+  set_overview_visible (self, self->is_open, ANIMATION_NONE);
 
   if (!self->is_open) {
     adw_tab_view_close_overview (self->view);
@@ -1002,6 +1032,14 @@ open_animation_done_cb (AdwTabOverview *self)
   gtk_widget_queue_draw (GTK_WIDGET (self));
 }
 
+static inline double
+inverse_lerp (double a,
+              double b,
+              double t)
+{
+  return (t - a) / (b - a);
+}
+
 static void
 calculate_bounds (AdwTabOverview  *self,
                   graphene_rect_t *bounds,
@@ -1051,11 +1089,18 @@ calculate_bounds (AdwTabOverview  *self,
                       adw_lerp (1, thumbnail_bounds.size.width / view_bounds.size.width, self->progress),
                       adw_lerp (1, thumbnail_bounds.size.height / view_bounds.size.height, self->progress));
 
-  graphene_rect_init (transition_bounds,
-                      adw_lerp (0, thumbnail_bounds.origin.x, self->progress),
-                      adw_lerp (0, thumbnail_bounds.origin.y, self->progress),
+  graphene_size_init (&transition_bounds->size,
                       clip_bounds->size.width * clip_scale->width,
                       clip_bounds->size.height * clip_scale->height);
+  graphene_point_init (&transition_bounds->origin,
+                       adw_lerp (0, thumbnail_bounds.origin.x,
+                                 inverse_lerp (bounds->size.width,
+                                               thumbnail_bounds.size.width,
+                                               transition_bounds->size.width)),
+                       adw_lerp (0, thumbnail_bounds.origin.y,
+                                 inverse_lerp (bounds->size.height,
+                                               thumbnail_bounds.size.height,
+                                               transition_bounds->size.height)));
 }
 
 static void
@@ -1373,6 +1418,9 @@ adw_tab_overview_get_property (GObject    *object,
   case PROP_SHOW_END_TITLE_BUTTONS:
     g_value_set_boolean (value, adw_tab_overview_get_show_end_title_buttons (self));
     break;
+  case PROP_EXTRA_DRAG_PREFERRED_ACTION:
+    g_value_set_flags (value, adw_tab_overview_get_extra_drag_preferred_action (self));
+    break;
   case PROP_EXTRA_DRAG_PRELOAD:
     g_value_set_boolean (value, adw_tab_overview_get_extra_drag_preload (self));
     break;
@@ -1631,6 +1679,23 @@ adw_tab_overview_class_init (AdwTabOverviewClass *klass)
                           TRUE,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
+  /**
+   * AdwTabOverview:extra-drag-preferred-action: (attributes org.gtk.Property.get=adw_tab_overview_get_extra_drag_preferred_action)
+   *
+   * The unique action on the `current-drop` of the
+   * [signal@TabOverview::extra-drag-drop].
+   *
+   * This property should only be used during a
+   * [signal@TabOverview::extra-drag-drop] and is always a subset of what was
+   * originally passed to [method@TabOverview.setup_extra_drop_target].
+   *
+   * Since: 1.4
+   */
+  props[PROP_EXTRA_DRAG_PREFERRED_ACTION] =
+    g_param_spec_flags ("extra-drag-preferred-action", NULL, NULL,
+                       GDK_TYPE_DRAG_ACTION, 0,
+                       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
   /**
    * AdwTabOverview:extra-drag-preload: (attributes org.gtk.Property.get=adw_tab_overview_get_extra_drag_preload org.gtk.Property.set=adw_tab_overview_set_extra_drag_preload)
    *
@@ -1772,6 +1837,9 @@ adw_tab_overview_class_init (AdwTabOverviewClass *klass)
   gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BIN_LAYOUT);
   gtk_widget_class_set_css_name (widget_class, "taboverview");
 
+  g_signal_override_class_handler ("extra-drag-value", G_TYPE_FROM_CLASS (klass),
+                                   G_CALLBACK (extra_drag_value_notify));
+
   g_type_ensure (ADW_TYPE_TAB_GRID);
   g_type_ensure (ADW_TYPE_TAB_OVERVIEW_SCROLLABLE);
 }
@@ -1972,6 +2040,9 @@ adw_tab_overview_set_child (AdwTabOverview *self,
   g_return_if_fail (ADW_IS_TAB_OVERVIEW (self));
   g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
 
+  if (child)
+    g_return_if_fail (gtk_widget_get_parent (child) == NULL);
+
   if (child == adw_tab_overview_get_child (self))
     return;
 
@@ -2071,11 +2142,11 @@ adw_tab_overview_set_open (AdwTabOverview *self,
 
     adw_tab_view_open_overview (self->view);
 
-    set_overview_visible (self, self->is_open, TRUE);
+    set_overview_visible (self, self->is_open, ANIMATION_IN);
 
     adw_tab_grid_try_focus_selected_tab (grid, FALSE);
   } else {
-    set_overview_visible (self, self->is_open, TRUE);
+    set_overview_visible (self, self->is_open, ANIMATION_OUT);
   }
 
   if (self->transition_picture)
@@ -2465,6 +2536,24 @@ adw_tab_overview_get_pinned_tab_grid (AdwTabOverview *self)
   return self->pinned_grid;
 }
 
+/**
+ * adw_tab_overview_get_extra_drag_preferred_action:
+ * @self: a tab overview
+ *
+ * Gets the current action during a drop on the extra_drop_target.
+ *
+ * Returns: the drag action of the current drop.
+ *
+ * Since: 1.4
+ */
+GdkDragAction
+adw_tab_overview_get_extra_drag_preferred_action (AdwTabOverview *self)
+{
+  g_return_val_if_fail (ADW_IS_TAB_OVERVIEW (self), 0);
+
+  return self->extra_drag_preferred_action;
+}
+
 /**
  * adw_tab_overview_get_extra_drag_preload: (attributes org.gtk.Method.get_property=extra-drag-preload)
  * @self: a tab overview
