@@ -3,7 +3,7 @@
  *
  * SPDX-License-Identifier: LGPL-2.1-or-later
  *
- * Author: Alexander Mikhaylenko <alexander.mikhaylenko@puri.sm>
+ * Author: Alice Mikhaylenko <alice.mikhaylenko@puri.sm>
  */
 
 #pragma once
@@ -28,10 +28,6 @@ AdwTabPage *adw_tab_get_page (AdwTab     *self);
 void        adw_tab_set_page (AdwTab     *self,
                               AdwTabPage *page);
 
-gboolean adw_tab_get_hovering (AdwTab   *self);
-void     adw_tab_set_hovering (AdwTab   *self,
-                               gboolean  hovering);
-
 gboolean adw_tab_get_dragging (AdwTab   *self);
 void     adw_tab_set_dragging (AdwTab   *self,
                                gboolean  dragging);
@@ -48,4 +44,7 @@ void adw_tab_setup_extra_drop_target (AdwTab        *self,
                                       GType         *types,
                                       gsize          n_types);
 
+void adw_tab_set_extra_drag_preload (AdwTab   *self,
+                                     gboolean  preload);
+
 G_END_DECLS
