@@ -10,7 +10,6 @@
 
 #include "adw-animation-private.h"
 #include "adw-animation-util.h"
-#include "adw-macros-private.h"
 
 /**
  * AdwTimedAnimation:
@@ -31,8 +30,6 @@
  * on the [property@TimedAnimation:repeat-count] value. If
  * [property@TimedAnimation:alternate] is set to `TRUE`, it will also change the
  * direction every other iteration.
- *
- * Since: 1.0
  */
 
 struct _AdwTimedAnimation
@@ -220,17 +217,13 @@ adw_timed_animation_class_init (AdwTimedAnimationClass *klass)
    *
    * If [property@TimedAnimation:reverse] is `TRUE`, the animation will end at
    * this value instead.
-   *
-   * Since: 1.0
    */
   props[PROP_VALUE_FROM] =
-    g_param_spec_double ("value-from",
-                         "Initial value",
-                         "The value to animate from",
+    g_param_spec_double ("value-from", NULL, NULL,
                          -G_MAXDOUBLE,
                          G_MAXDOUBLE,
                          0,
-                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwTimedAnimation:value-to: (attributes org.gtk.Property.get=adw_timed_animation_get_value_to org.gtk.Property.set=adw_timed_animation_set_value_to)
@@ -242,17 +235,13 @@ adw_timed_animation_class_init (AdwTimedAnimationClass *klass)
    *
    * If [property@TimedAnimation:reverse] is `TRUE`, the animation will start
    * at this value instead.
-   *
-   * Since: 1.0
    */
   props[PROP_VALUE_TO] =
-    g_param_spec_double ("value-to",
-                         "Final value",
-                         "The value to animate to",
+    g_param_spec_double ("value-to", NULL, NULL,
                          -G_MAXDOUBLE,
                          G_MAXDOUBLE,
                          0,
-                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwTimedAnimation:duration: (attributes org.gtk.Property.get=adw_timed_animation_get_duration org.gtk.Property.set=adw_timed_animation_set_duration)
@@ -263,17 +252,13 @@ adw_timed_animation_class_init (AdwTimedAnimationClass *klass)
    *
    * If the animation repeats more than once, describes the duration of one
    * iteration.
-   *
-   * Since: 1.0
    */
   props[PROP_DURATION] =
-    g_param_spec_uint ("duration",
-                       "Duration",
-                       "Duration of the animation in milliseconds",
+    g_param_spec_uint ("duration", NULL, NULL,
                        0,
                        ADW_DURATION_INFINITE,
                        0,
-                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwTimedAnimation:easing: (attributes org.gtk.Property.get=adw_timed_animation_get_easing org.gtk.Property.set=adw_timed_animation_set_easing)
@@ -283,16 +268,12 @@ adw_timed_animation_class_init (AdwTimedAnimationClass *klass)
    * Describes the curve the value is interpolated on.
    *
    * See [enum@Easing] for the description of specific easing functions.
-   *
-   * Since: 1.0
    */
   props[PROP_EASING] =
-    g_param_spec_enum ("easing",
-                       "Easing",
-                       "Easing function used in the animation",
+    g_param_spec_enum ("easing", NULL, NULL,
                        ADW_TYPE_EASING,
                        ADW_EASE_OUT_CUBIC,
-                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwTimedAnimation:repeat-count: (attributes org.gtk.Property.get=adw_timed_animation_get_repeat_count org.gtk.Property.set=adw_timed_animation_set_repeat_count)
@@ -300,45 +281,33 @@ adw_timed_animation_class_init (AdwTimedAnimationClass *klass)
    * Number of times the animation will play.
    *
    * If set to 0, the animation will repeat endlessly.
-   *
-   * Since: 1.0
    */
   props[PROP_REPEAT_COUNT] =
-    g_param_spec_uint ("repeat-count",
-                       "Repeat count",
-                       "Number of times the animation will play",
+    g_param_spec_uint ("repeat-count", NULL, NULL,
                        0,
                        G_MAXUINT,
                        1,
-                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+                       G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwTimedAnimation:reverse: (attributes org.gtk.Property.get=adw_timed_animation_get_reverse org.gtk.Property.set=adw_timed_animation_set_reverse)
    *
    * Whether the animation plays backwards.
-   *
-   * Since: 1.0
    */
   props[PROP_REVERSE] =
-    g_param_spec_boolean ("reverse",
-                          "Reverse",
-                          "Whether the animation plays backwards",
+    g_param_spec_boolean ("reverse", NULL, NULL,
                           FALSE,
-                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwTimedAnimation:alternate: (attributes org.gtk.Property.get=adw_timed_animation_get_alternate org.gtk.Property.set=adw_timed_animation_set_alternate)
    *
    * Whether the animation changes direction on every iteration.
-   *
-   * Since: 1.0
    */
   props[PROP_ALTERNATE] =
-    g_param_spec_boolean ("alternate",
-                          "Alternate",
-                          "Whether the animation changes direction on every iteration",
+    g_param_spec_boolean ("alternate", NULL, NULL,
                           FALSE,
-                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   g_object_class_install_properties (object_class, LAST_PROP, props);
 }
@@ -360,8 +329,6 @@ adw_timed_animation_init (AdwTimedAnimation *self)
  * to @to.
  *
  * Returns: (transfer none): the newly created animation
- *
- * Since: 1.0
  */
 AdwAnimation *
 adw_timed_animation_new (GtkWidget          *widget,
@@ -395,8 +362,6 @@ adw_timed_animation_new (GtkWidget          *widget,
  * Gets the value @self will animate from.
  *
  * Returns: the value to animate from
- *
- * Since: 1.0
  */
 double
 adw_timed_animation_get_value_from (AdwTimedAnimation *self)
@@ -413,7 +378,11 @@ adw_timed_animation_get_value_from (AdwTimedAnimation *self)
  *
  * Sets the value @self will animate from.
  *
- * Since: 1.0
+ * The animation will start at this value and end at
+ * [property@TimedAnimation:value-to].
+ *
+ * If [property@TimedAnimation:reverse] is `TRUE`, the animation will end at
+ * this value instead.
  */
 void
 adw_timed_animation_set_value_from (AdwTimedAnimation *self,
@@ -421,7 +390,7 @@ adw_timed_animation_set_value_from (AdwTimedAnimation *self,
 {
   g_return_if_fail (ADW_IS_TIMED_ANIMATION (self));
 
-  if (self->value_from == value)
+  if (G_APPROX_VALUE (self->value_from, value, DBL_EPSILON))
     return;
 
   self->value_from = value;
@@ -436,8 +405,6 @@ adw_timed_animation_set_value_from (AdwTimedAnimation *self,
  * Gets the value @self will animate to.
  *
  * Returns: the value to animate to
- *
- * Since: 1.0
  */
 double
 adw_timed_animation_get_value_to (AdwTimedAnimation *self)
@@ -454,7 +421,11 @@ adw_timed_animation_get_value_to (AdwTimedAnimation *self)
  *
  * Sets the value @self will animate to.
  *
- * Since: 1.0
+ * The animation will start at [property@TimedAnimation:value-from] and end at
+ * this value.
+ *
+ * If [property@TimedAnimation:reverse] is `TRUE`, the animation will start
+ * at this value instead.
  */
 void
 adw_timed_animation_set_value_to (AdwTimedAnimation *self,
@@ -462,7 +433,7 @@ adw_timed_animation_set_value_to (AdwTimedAnimation *self,
 {
   g_return_if_fail (ADW_IS_TIMED_ANIMATION (self));
 
-  if (self->value_to == value)
+  if (G_APPROX_VALUE (self->value_to, value, DBL_EPSILON))
     return;
 
   self->value_to = value;
@@ -477,8 +448,6 @@ adw_timed_animation_set_value_to (AdwTimedAnimation *self,
  * Gets the duration of @self.
  *
  * Returns: the duration of @self, in milliseconds
- *
- * Since: 1.0
  */
 guint
 adw_timed_animation_get_duration (AdwTimedAnimation *self)
@@ -496,8 +465,6 @@ adw_timed_animation_get_duration (AdwTimedAnimation *self)
  * Sets the duration of @self.
  *
  * If the animation repeats more than once, sets the duration of one iteration.
- *
- * Since: 1.0
  */
 void
 adw_timed_animation_set_duration (AdwTimedAnimation *self,
@@ -520,8 +487,6 @@ adw_timed_animation_set_duration (AdwTimedAnimation *self,
  * Gets the easing function @self uses.
  *
  * Returns: the easing function @self uses
- *
- * Since: 1.0
  */
 AdwEasing
 adw_timed_animation_get_easing (AdwTimedAnimation *self)
@@ -540,8 +505,6 @@ adw_timed_animation_get_easing (AdwTimedAnimation *self)
  * Sets the easing function @self will use.
  *
  * See [enum@Easing] for the description of specific easing functions.
- *
- * Since: 1.0
  */
 void
 adw_timed_animation_set_easing (AdwTimedAnimation *self,
@@ -565,8 +528,6 @@ adw_timed_animation_set_easing (AdwTimedAnimation *self,
  * Gets the number of times @self will play.
  *
  * Returns: the number of times @self will play
- *
- * Since: 1.0
  */
 guint
 adw_timed_animation_get_repeat_count (AdwTimedAnimation *self)
@@ -584,8 +545,6 @@ adw_timed_animation_get_repeat_count (AdwTimedAnimation *self)
  * Sets the number of times @self will play.
  *
  * If set to 0, @self will repeat endlessly.
- *
- * Since: 1.0
  */
 void
 adw_timed_animation_set_repeat_count (AdwTimedAnimation *self,
@@ -608,8 +567,6 @@ adw_timed_animation_set_repeat_count (AdwTimedAnimation *self,
  * Gets whether @self plays backwards.
  *
  * Returns: whether @self plays backwards
- *
- * Since: 1.0
  */
 gboolean
 adw_timed_animation_get_reverse (AdwTimedAnimation *self)
@@ -625,8 +582,6 @@ adw_timed_animation_get_reverse (AdwTimedAnimation *self)
  * @reverse: whether @self plays backwards
  *
  * Sets whether @self plays backwards.
- *
- * Since: 1.0
  */
 void
 adw_timed_animation_set_reverse (AdwTimedAnimation *self,
@@ -649,8 +604,6 @@ adw_timed_animation_set_reverse (AdwTimedAnimation *self,
  * Gets whether @self changes direction on every iteration.
  *
  * Returns: whether @self alternates
- *
- * Since: 1.0
  */
 gboolean
 adw_timed_animation_get_alternate (AdwTimedAnimation *self)
@@ -666,8 +619,6 @@ adw_timed_animation_get_alternate (AdwTimedAnimation *self)
  * @alternate: whether @self alternates
  *
  * Sets whether @self changes direction on every iteration.
- *
- * Since: 1.0
  */
 void
 adw_timed_animation_set_alternate (AdwTimedAnimation *self,
