@@ -16,7 +16,6 @@
 
 #include "adw-animation-util.h"
 #include "adw-gizmo-private.h"
-#include "adw-macros-private.h"
 #include "adw-widget-utils-private.h"
 
 /**
@@ -72,15 +71,26 @@
  *
  * `AdwViewStack` has a single CSS node named `stack`.
  *
- * Since: 1.0
+ * ## Accessibility
+ *
+ * `AdwViewStack` uses the `GTK_ACCESSIBLE_ROLE_TAB_PANEL` for the stack pages
+ * which are the accessible parent objects of the child widgets.
  */
 
 /**
  * AdwViewStackPage:
  *
  * An auxiliary class used by [class@ViewStack].
+ */
+
+/**
+ * AdwViewStackPages:
  *
- * Since: 1.0
+ * An auxiliary class used by [class@ViewStack].
+ *
+ * See [property@ViewStack:pages].
+ *
+ * Since: 1.4
  */
 
 #define OPPOSITE_ORIENTATION(_orientation) (1 - (_orientation))
@@ -105,12 +115,19 @@ struct _AdwViewStackPage {
   guint badge_number;
   GtkWidget *last_focus;
 
-  bool needs_attention;
-  bool visible;
-  bool use_underline;
+  GtkATContext *at_context;
+  AdwViewStackPage *next_page;
+
+  gboolean needs_attention;
+  gboolean visible;
+  gboolean use_underline;
+  gboolean in_destruction;
 };
 
-G_DEFINE_FINAL_TYPE (AdwViewStackPage, adw_view_stack_page, G_TYPE_OBJECT)
+static void adw_view_stack_page_accessible_init (GtkAccessibleInterface *iface);
+
+G_DEFINE_FINAL_TYPE_WITH_CODE (AdwViewStackPage, adw_view_stack_page, G_TYPE_OBJECT,
+                               G_IMPLEMENT_INTERFACE (GTK_TYPE_ACCESSIBLE, adw_view_stack_page_accessible_init))
 
 enum {
   PAGE_PROP_0,
@@ -122,11 +139,26 @@ enum {
   PAGE_PROP_NEEDS_ATTENTION,
   PAGE_PROP_BADGE_NUMBER,
   PAGE_PROP_VISIBLE,
-  LAST_PAGE_PROP
+  LAST_PAGE_PROP,
+  PAGE_PROP_ACCESSIBLE_ROLE
 };
 
 static GParamSpec *page_props[LAST_PAGE_PROP];
 
+struct _AdwViewStackPages
+{
+  GObject parent_instance;
+  AdwViewStack *stack;
+};
+
+enum {
+  PAGES_PROP_0,
+  PAGES_PROP_SELECTED_PAGE,
+  N_PAGES_PROPS,
+};
+
+static GParamSpec *pages_props[N_PAGES_PROPS];
+
 struct _AdwViewStack {
   GtkWidget parent_instance;
 
@@ -142,9 +174,11 @@ struct _AdwViewStack {
 static GParamSpec *props[LAST_PROP];
 
 static void adw_view_stack_buildable_init (GtkBuildableIface *iface);
+static void adw_view_stack_accessible_init (GtkAccessibleInterface *iface);
 
 G_DEFINE_FINAL_TYPE_WITH_CODE (AdwViewStack, adw_view_stack, GTK_TYPE_WIDGET,
-                               G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE, adw_view_stack_buildable_init))
+                               G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE, adw_view_stack_buildable_init)
+                               G_IMPLEMENT_INTERFACE (GTK_TYPE_ACCESSIBLE, adw_view_stack_accessible_init))
 
 static GtkBuildableIface *parent_buildable_iface;
 
@@ -181,6 +215,9 @@ adw_view_stack_page_get_property (GObject      *object,
   case PAGE_PROP_VISIBLE:
     g_value_set_boolean (value, adw_view_stack_page_get_visible (self));
     break;
+  case PAGE_PROP_ACCESSIBLE_ROLE:
+    g_value_set_enum (value, GTK_ACCESSIBLE_ROLE_TAB_PANEL);
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     break;
@@ -220,12 +257,26 @@ adw_view_stack_page_set_property (GObject      *object,
   case PAGE_PROP_VISIBLE:
     adw_view_stack_page_set_visible (self, g_value_get_boolean (value));
     break;
+  case PAGE_PROP_ACCESSIBLE_ROLE:
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     break;
   }
 }
 
+static void
+adw_view_stack_page_dispose (GObject *object)
+{
+  AdwViewStackPage *self = ADW_VIEW_STACK_PAGE (object);
+
+  self->in_destruction = TRUE;
+
+  g_clear_object (&self->at_context);
+
+  G_OBJECT_CLASS (adw_view_stack_page_parent_class)->dispose (object);
+}
+
 static void
 adw_view_stack_page_finalize (GObject *object)
 {
@@ -248,6 +299,7 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (class);
 
+  object_class->dispose = adw_view_stack_page_dispose;
   object_class->finalize = adw_view_stack_page_finalize;
   object_class->get_property = adw_view_stack_page_get_property;
   object_class->set_property = adw_view_stack_page_set_property;
@@ -256,8 +308,6 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
    * AdwViewStackPage:child: (attributes org.gtk.Property.get=adw_view_stack_page_get_child)
    *
    * The stack child to which the page belongs.
-   *
-   * Since: 1.0
    */
   page_props[PAGE_PROP_CHILD] =
     g_param_spec_object ("child", NULL, NULL,
@@ -268,8 +318,6 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
    * AdwViewStackPage:name: (attributes org.gtk.Property.get=adw_view_stack_page_get_name org.gtk.Property.set=adw_view_stack_page_set_name)
    *
    * The name of the child page.
-   *
-   * Since: 1.0
    */
   page_props[PAGE_PROP_NAME] =
     g_param_spec_string ("name", NULL, NULL,
@@ -280,8 +328,6 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
    * AdwViewStackPage:title: (attributes org.gtk.Property.get=adw_view_stack_page_get_title org.gtk.Property.set=adw_view_stack_page_set_title)
    *
    * The title of the child page.
-   *
-   * Since: 1.0
    */
   page_props[PAGE_PROP_TITLE] =
     g_param_spec_string ("title", NULL, NULL,
@@ -292,8 +338,6 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
    * AdwViewStackPage:use-underline: (attributes org.gtk.Property.get=adw_view_stack_page_get_use_underline org.gtk.Property.set=adw_view_stack_page_set_use_underline)
    *
    * Whether an embedded underline in the title indicates a mnemonic.
-   *
-   * Since: 1.0
    */
   page_props[PAGE_PROP_USE_UNDERLINE] =
     g_param_spec_boolean ("use-underline", NULL, NULL,
@@ -304,8 +348,6 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
    * AdwViewStackPage:icon-name: (attributes org.gtk.Property.get=adw_view_stack_page_get_icon_name org.gtk.Property.set=adw_view_stack_page_set_icon_name)
    *
    * The icon name of the child page.
-   *
-   * Since: 1.0
    */
   page_props[PAGE_PROP_ICON_NAME] =
     g_param_spec_string ("icon-name", NULL, NULL,
@@ -318,8 +360,6 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
    * Whether the page requires the user attention.
    *
    * [class@ViewSwitcher] will display it as a dot next to the page icon.
-   *
-   * Since: 1.0
    */
   page_props[PAGE_PROP_NEEDS_ATTENTION] =
     g_param_spec_boolean ("needs-attention", NULL, NULL,
@@ -335,8 +375,6 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
    * commonly used to display a number of unread items within the page.
    *
    * It can be used together with [property@ViewStack{age}:needs-attention].
-   *
-   * Since: 1.0
    */
   page_props[PAGE_PROP_BADGE_NUMBER] =
     g_param_spec_uint ("badge-number", NULL, NULL,
@@ -350,8 +388,6 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
    *
    * This is independent from the [property@Gtk.Widget:visible] property of
    * [property@ViewStackPage:child].
-   *
-   * Since: 1.0
    */
   page_props[PAGE_PROP_VISIBLE] =
     g_param_spec_boolean ("visible", NULL, NULL,
@@ -359,6 +395,8 @@ adw_view_stack_page_class_init (AdwViewStackPageClass *class)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   g_object_class_install_properties (object_class, LAST_PAGE_PROP, page_props);
+
+  g_object_class_override_property (object_class, PAGE_PROP_ACCESSIBLE_ROLE, "accessible-role");
 }
 
 static void
@@ -367,15 +405,100 @@ adw_view_stack_page_init (AdwViewStackPage *self)
   self->visible = TRUE;
 }
 
-#define ADW_TYPE_VIEW_STACK_PAGES (adw_view_stack_pages_get_type ())
-
-G_DECLARE_FINAL_TYPE (AdwViewStackPages, adw_view_stack_pages, ADW, VIEW_STACK_PAGES, GObject)
-
-struct _AdwViewStackPages
+static GtkATContext *
+adw_view_stack_page_accessible_get_at_context (GtkAccessible *accessible)
 {
-  GObject parent_instance;
-  AdwViewStack *stack;
-};
+  AdwViewStackPage *self = ADW_VIEW_STACK_PAGE (accessible);
+
+  if (self->in_destruction)
+    return NULL;
+
+  if (self->at_context == NULL) {
+    GtkAccessibleRole role = GTK_ACCESSIBLE_ROLE_TAB_PANEL;
+    GdkDisplay *display;
+
+    if (self->widget != NULL)
+      display = gtk_widget_get_display (self->widget);
+    else
+      display = gdk_display_get_default ();
+
+    self->at_context = gtk_at_context_create (role, accessible, display);
+
+    if (self->at_context == NULL)
+      return NULL;
+  }
+
+  return g_object_ref (self->at_context);
+}
+
+static gboolean
+adw_view_stack_page_accessible_get_platform_state (GtkAccessible              *self,
+                                                   GtkAccessiblePlatformState  state)
+{
+  return FALSE;
+}
+
+static GtkAccessible *
+adw_view_stack_page_accessible_get_accessible_parent (GtkAccessible *accessible)
+{
+  AdwViewStackPage *self = ADW_VIEW_STACK_PAGE (accessible);
+  GtkWidget *parent;
+
+  if (!self->widget)
+    return NULL;
+
+  parent = gtk_widget_get_parent (self->widget);
+
+  return GTK_ACCESSIBLE (g_object_ref (parent));
+}
+
+static GtkAccessible *
+adw_view_stack_page_accessible_get_first_accessible_child (GtkAccessible *accessible)
+{
+  AdwViewStackPage *self = ADW_VIEW_STACK_PAGE (accessible);
+
+  if (!self->widget)
+    return NULL;
+
+  return GTK_ACCESSIBLE (g_object_ref (self->widget));
+}
+
+static GtkAccessible *
+adw_view_stack_page_accessible_get_next_accessible_sibling (GtkAccessible *accessible)
+{
+  AdwViewStackPage *self = ADW_VIEW_STACK_PAGE (accessible);
+
+  if (!self->next_page)
+    return NULL;
+
+  return GTK_ACCESSIBLE (g_object_ref (self->next_page));
+}
+
+static gboolean
+adw_view_stack_page_accessible_get_bounds (GtkAccessible *accessible,
+                                           int           *x,
+                                           int           *y,
+                                           int           *width,
+                                           int           *height)
+{
+  AdwViewStackPage *self = ADW_VIEW_STACK_PAGE (accessible);
+
+  if (self->widget)
+    return gtk_accessible_get_bounds (GTK_ACCESSIBLE (self->widget), x, y, width, height);
+
+  return FALSE;
+}
+
+static void
+adw_view_stack_page_accessible_init (GtkAccessibleInterface *iface)
+{
+  iface->get_at_context = adw_view_stack_page_accessible_get_at_context;
+  iface->get_platform_state = adw_view_stack_page_accessible_get_platform_state;
+  iface->get_accessible_parent = adw_view_stack_page_accessible_get_accessible_parent;
+  iface->get_first_accessible_child = adw_view_stack_page_accessible_get_first_accessible_child;
+  iface->get_next_accessible_sibling = adw_view_stack_page_accessible_get_next_accessible_sibling;
+  iface->get_bounds = adw_view_stack_page_accessible_get_bounds;
+}
 
 static GType
 adw_view_stack_pages_get_item_type (GListModel *model)
@@ -452,6 +575,102 @@ G_DEFINE_FINAL_TYPE_WITH_CODE (AdwViewStackPages, adw_view_stack_pages, G_TYPE_O
                                G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL, adw_view_stack_pages_list_model_init)
                                G_IMPLEMENT_INTERFACE (GTK_TYPE_SELECTION_MODEL, adw_view_stack_pages_selection_model_init))
 
+/**
+ * adw_view_stack_pages_get_selected_page: (attributes org.gtk.Method.get_property=selected-page)
+ * @self: a [class@ViewStackPages]
+ *
+ * Gets the [class@ViewStackPage] for the visible child of a view stack
+ *
+ * Gets the [class@ViewStackPage] for the visible child of the associated stack.
+ *
+ * Returns `NULL` if there's no selected page.
+ *
+ * Returns: (transfer none) (nullable): the stack page
+ *
+ * Since: 1.4
+ */
+AdwViewStackPage *
+adw_view_stack_pages_get_selected_page (AdwViewStackPages *self)
+{
+  GtkWidget *child;
+
+  g_return_val_if_fail (ADW_IS_VIEW_STACK_PAGES (self), NULL);
+
+  if (self->stack == NULL)
+    return NULL;
+
+  if ((child = adw_view_stack_get_visible_child (self->stack)))
+    return adw_view_stack_get_page (self->stack, child);
+
+  return NULL;
+}
+
+/**
+ * adw_view_stack_pages_set_selected_page: (attributes org.gtk.Method.set_property=selected-page)
+ * @self: a [class@ViewStackPages]
+ * @page: a stack page within the associated stack
+ *
+ * Sets the visible child in the associated [class@ViewStack].
+ *
+ * See [property@ViewStack:visible-child].
+ *
+ * Since: 1.4
+ */
+void
+adw_view_stack_pages_set_selected_page (AdwViewStackPages *self,
+                                        AdwViewStackPage  *page)
+{
+  GtkWidget *child = NULL;
+
+  g_return_if_fail (ADW_IS_VIEW_STACK_PAGES (self));
+  g_return_if_fail (!page || ADW_IS_VIEW_STACK_PAGE (page));
+
+  if (self->stack == NULL)
+    return;
+
+  if (page == adw_view_stack_pages_get_selected_page (self))
+    return;
+
+  if (page != NULL)
+    child = adw_view_stack_page_get_child (page);
+
+  adw_view_stack_set_visible_child (self->stack, child);
+}
+
+static void
+adw_view_stack_pages_get_property (GObject    *object,
+                                   guint       prop_id,
+                                   GValue     *value,
+                                   GParamSpec *pspec)
+{
+  AdwViewStackPages *self = ADW_VIEW_STACK_PAGES (object);
+
+  switch (prop_id) {
+  case PAGES_PROP_SELECTED_PAGE:
+    g_value_set_object (value, adw_view_stack_pages_get_selected_page (self));
+    break;
+  default:
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+  }
+}
+
+static void
+adw_view_stack_pages_set_property (GObject      *object,
+                                   guint         prop_id,
+                                   const GValue *value,
+                                   GParamSpec   *pspec)
+{
+  AdwViewStackPages *self = ADW_VIEW_STACK_PAGES (object);
+
+  switch (prop_id) {
+  case PAGES_PROP_SELECTED_PAGE:
+    adw_view_stack_pages_set_selected_page (self, g_value_get_object (value));
+    break;
+  default:
+    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+  }
+}
+
 static void
 adw_view_stack_pages_init (AdwViewStackPages *pages)
 {
@@ -460,6 +679,29 @@ adw_view_stack_pages_init (AdwViewStackPages *pages)
 static void
 adw_view_stack_pages_class_init (AdwViewStackPagesClass *class)
 {
+  GObjectClass *object_class = G_OBJECT_CLASS (class);
+
+  object_class->get_property = adw_view_stack_pages_get_property;
+  object_class->set_property = adw_view_stack_pages_set_property;
+
+  /**
+   * AdwViewStackPages:selected-page: (attributes org.gtk.Property.get=adw_view_stack_pages_get_selected_page org.gtk.Property.set=adw_view_stack_pages_set_selected_page)
+   *
+   * The selected [class@ViewStackPage] within the [class@ViewStackPages].
+   *
+   * This can be used to keep an up-to-date view of the [class@ViewStackPage] for
+   * The visible [class@ViewStackPage] within the associated [class@ViewStackPages].
+   *
+   * This can be used to keep an up-to-date view of the visible child.
+   *
+   * Since: 1.4
+   */
+  pages_props[PAGES_PROP_SELECTED_PAGE] =
+    g_param_spec_object ("selected-page", NULL, NULL,
+                         ADW_TYPE_VIEW_STACK_PAGE,
+                         (G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS));
+
+  g_object_class_install_properties (object_class, N_PAGES_PROPS, pages_props);
 }
 
 static AdwViewStackPages *
@@ -473,8 +715,8 @@ adw_view_stack_pages_new (AdwViewStack *stack)
   return pages;
 }
 
-static AdwViewStackPage *
-find_page_for_widget (AdwViewStack *self,
+static GList *
+find_link_for_widget (AdwViewStack *self,
                       GtkWidget    *child)
 {
   AdwViewStackPage *page;
@@ -484,12 +726,24 @@ find_page_for_widget (AdwViewStack *self,
     page = l->data;
 
     if (page->widget == child)
-      return page;
+      return l;
   }
 
   return NULL;
 }
 
+static AdwViewStackPage *
+find_page_for_widget (AdwViewStack *self,
+                      GtkWidget    *child)
+{
+  GList *l = find_link_for_widget (self, child);
+
+  if (l)
+    return l->data;
+
+  return NULL;
+}
+
 static AdwViewStackPage *
 find_page_for_name (AdwViewStack *self,
                     const char   *name)
@@ -599,6 +853,9 @@ set_visible_child (AdwViewStack     *self,
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_VISIBLE_CHILD]);
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_VISIBLE_CHILD_NAME]);
 
+  if (self->pages != NULL)
+    g_object_notify_by_pspec (G_OBJECT (self->pages), pages_props[PAGES_PROP_SELECTED_PAGE]);
+
   if (self->pages) {
     if (old_pos == GTK_INVALID_LIST_POSITION && new_pos == GTK_INVALID_LIST_POSITION)
       ; /* nothing to do */
@@ -625,6 +882,10 @@ update_child_visible (AdwViewStack     *self,
     set_visible_child (self, page);
   else if (self->visible_child == page && !visible)
     set_visible_child (self, NULL);
+
+  gtk_accessible_update_state (GTK_ACCESSIBLE (page),
+                               GTK_ACCESSIBLE_STATE_HIDDEN, !visible,
+                               -1);
 }
 
 static void
@@ -660,6 +921,14 @@ add_page (AdwViewStack     *self,
     }
   }
 
+  if (self->children) {
+    AdwViewStackPage *prev_last = g_list_last (self->children)->data;
+
+    prev_last->next_page = page;
+  } else {
+    page->next_page = NULL;
+  }
+
   self->children = g_list_append (self->children, g_object_ref (page));
 
   gtk_widget_set_child_visible (page->widget, FALSE);
@@ -712,11 +981,14 @@ stack_remove (AdwViewStack  *self,
 {
   AdwViewStackPage *page;
   gboolean was_visible;
+  GList *l;
 
-  page = find_page_for_widget (self, child);
-  if (!page)
+  l = find_link_for_widget (self, child);
+  if (!l)
     return;
 
+  page = l->data;
+
   g_signal_handlers_disconnect_by_func (child,
                                         stack_child_visibility_notify_cb,
                                         self);
@@ -730,8 +1002,16 @@ stack_remove (AdwViewStack  *self,
 
   g_clear_object (&page->widget);
 
+  l = l->prev;
+
   self->children = g_list_remove (self->children, page);
 
+  if (l) {
+    AdwViewStackPage *prev_page = l->data;
+
+    prev_page->next_page = page->next_page;
+  }
+
   g_object_unref (page);
 
   if (!in_dispose &&
@@ -905,8 +1185,6 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
    *
    * If it's `FALSE`, the stack may change width when a different child becomes
    * visible.
-   *
-   * Since: 1.0
    */
   props[PROP_HHOMOGENEOUS] =
     g_param_spec_boolean ("hhomogeneous", NULL, NULL,
@@ -923,8 +1201,6 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
    *
    * If it's `FALSE`, the stack may change height when a different child becomes
    * visible.
-   *
-   * Since: 1.0
    */
   props[PROP_VHOMOGENEOUS] =
     g_param_spec_boolean ("vhomogeneous", NULL, NULL,
@@ -935,8 +1211,6 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
    * AdwViewStack:visible-child: (attributes org.gtk.Property.get=adw_view_stack_get_visible_child org.gtk.Property.set=adw_view_stack_set_visible_child)
    *
    * The widget currently visible in the stack.
-   *
-   * Since: 1.0
    */
   props[PROP_VISIBLE_CHILD] =
     g_param_spec_object ("visible-child", NULL, NULL,
@@ -949,8 +1223,6 @@ adw_view_stack_class_init (AdwViewStackClass *klass)
    * The name of the widget currently visible in the stack.
    *
    * See [property@ViewStack:visible-child].
-   *
-   * Since: 1.0
    */
   props[PROP_VISIBLE_CHILD_NAME] =
     g_param_spec_string ("visible-child-name", NULL, NULL,
@@ -1008,6 +1280,23 @@ adw_view_stack_buildable_init (GtkBuildableIface *iface)
   iface->add_child = adw_view_stack_buildable_add_child;
 }
 
+static GtkAccessible *
+adw_view_stack_accessible_get_first_accessible_child (GtkAccessible *accessible)
+{
+  AdwViewStack *self = ADW_VIEW_STACK (accessible);
+
+  if (self->children && self->children->data)
+    return g_object_ref (self->children->data);
+
+  return NULL;
+}
+
+static void
+adw_view_stack_accessible_init (GtkAccessibleInterface *iface)
+{
+  iface->get_first_accessible_child = adw_view_stack_accessible_get_first_accessible_child;
+}
+
 /**
  * adw_view_stack_page_get_child: (attributes org.gtk.Method.get_property=child)
  * @self: a view stack page
@@ -1015,8 +1304,6 @@ adw_view_stack_buildable_init (GtkBuildableIface *iface)
  * Gets the stack child to which @self belongs.
  *
  * Returns: (transfer none): the child to which @self belongs
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_view_stack_page_get_child (AdwViewStackPage *self)
@@ -1033,8 +1320,6 @@ adw_view_stack_page_get_child (AdwViewStackPage *self)
  * Gets the name of the page.
  *
  * Returns: (nullable): the name of the page
- *
- * Since: 1.0
  */
 const char *
 adw_view_stack_page_get_name (AdwViewStackPage *self)
@@ -1050,8 +1335,6 @@ adw_view_stack_page_get_name (AdwViewStackPage *self)
  * @name: (nullable): the page name
  *
  * Sets the name of the page.
- *
- * Since: 1.0
  */
 void
 adw_view_stack_page_set_name (AdwViewStackPage *self,
@@ -1081,12 +1364,9 @@ adw_view_stack_page_set_name (AdwViewStackPage *self,
     }
   }
 
-  if (name == self->name)
+  if (!g_set_str (&self->name, name))
     return;
 
-  g_free (self->name);
-  self->name = g_strdup (name);
-
   g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_NAME]);
 
   if (stack && stack->visible_child == self)
@@ -1101,8 +1381,6 @@ adw_view_stack_page_set_name (AdwViewStackPage *self,
  * Gets the page title.
  *
  * Returns: (nullable): the page title
- *
- * Since: 1.0
  */
 const char *
 adw_view_stack_page_get_title (AdwViewStackPage *self)
@@ -1118,8 +1396,6 @@ adw_view_stack_page_get_title (AdwViewStackPage *self)
  * @title: (nullable): the page title
  *
  * Sets the page title.
- *
- * Since: 1.0
  */
 void
 adw_view_stack_page_set_title (AdwViewStackPage *self,
@@ -1127,13 +1403,14 @@ adw_view_stack_page_set_title (AdwViewStackPage *self,
 {
   g_return_if_fail (ADW_IS_VIEW_STACK_PAGE (self));
 
-  if (title == self->title)
+  if (!g_set_str (&self->title, title))
     return;
 
-  g_free (self->title);
-  self->title = g_strdup (title);
-
   g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_TITLE]);
+
+  gtk_accessible_update_property (GTK_ACCESSIBLE (self),
+                                  GTK_ACCESSIBLE_PROPERTY_LABEL, self->title,
+                                  -1);
 }
 
 /**
@@ -1143,8 +1420,6 @@ adw_view_stack_page_set_title (AdwViewStackPage *self,
  * Gets whether underlines in the page title indicate mnemonics.
  *
  * Returns: whether underlines in the page title indicate mnemonics
- *
- * Since: 1.0
  */
 gboolean
 adw_view_stack_page_get_use_underline (AdwViewStackPage *self)
@@ -1158,8 +1433,6 @@ adw_view_stack_page_get_use_underline (AdwViewStackPage *self)
  * @use_underline: the new value to set
  *
  * Sets whether underlines in the page title indicate mnemonics.
- *
- * Since: 1.0
  */
 void
 adw_view_stack_page_set_use_underline (AdwViewStackPage *self,
@@ -1182,8 +1455,6 @@ adw_view_stack_page_set_use_underline (AdwViewStackPage *self,
  * Gets the icon name of the page.
  *
  * Returns: (nullable): the icon name of the page
- *
- * Since: 1.0
  */
 const char *
 adw_view_stack_page_get_icon_name (AdwViewStackPage *self)
@@ -1199,8 +1470,6 @@ adw_view_stack_page_get_icon_name (AdwViewStackPage *self)
  * @icon_name: (nullable): the icon name
  *
  * Sets the icon name of the page.
- *
- * Since: 1.0
  */
 void
 adw_view_stack_page_set_icon_name (AdwViewStackPage *self,
@@ -1208,12 +1477,9 @@ adw_view_stack_page_set_icon_name (AdwViewStackPage *self,
 {
   g_return_if_fail (ADW_IS_VIEW_STACK_PAGE (self));
 
-  if (icon_name == self->icon_name)
+  if (!g_set_str (&self->icon_name, icon_name))
     return;
 
-  g_free (self->icon_name);
-  self->icon_name = g_strdup (icon_name);
-
   g_object_notify_by_pspec (G_OBJECT (self), page_props[PAGE_PROP_ICON_NAME]);
 }
 
@@ -1224,8 +1490,6 @@ adw_view_stack_page_set_icon_name (AdwViewStackPage *self,
  * Gets whether the page requires the user attention.
  *
  * Returns: whether the page needs attention
- *
- * Since: 1.0
  */
 gboolean
 adw_view_stack_page_get_needs_attention (AdwViewStackPage *self)
@@ -1243,8 +1507,6 @@ adw_view_stack_page_get_needs_attention (AdwViewStackPage *self)
  * Sets whether the page requires the user attention.
  *
  * [class@ViewSwitcher] will display it as a dot next to the page icon.
- *
- * Since: 1.0
  */
 void
 adw_view_stack_page_set_needs_attention (AdwViewStackPage *self,
@@ -1269,8 +1531,6 @@ adw_view_stack_page_set_needs_attention (AdwViewStackPage *self,
  * Gets the badge number for this page.
  *
  * Returns: the badge number for this page
- *
- * Since: 1.0
  */
 guint
 adw_view_stack_page_get_badge_number (AdwViewStackPage *self)
@@ -1291,8 +1551,6 @@ adw_view_stack_page_get_badge_number (AdwViewStackPage *self)
  * commonly used to display a number of unread items within the page.
  *
  * It can be used together with [property@ViewStack{age}:needs-attention].
- *
- * Since: 1.0
  */
 void
 adw_view_stack_page_set_badge_number (AdwViewStackPage *self,
@@ -1318,8 +1576,6 @@ adw_view_stack_page_set_badge_number (AdwViewStackPage *self,
  * property of its widget.
  *
  * Returns: whether @self is visible
- *
- * Since: 1.0
  */
 gboolean
 adw_view_stack_page_get_visible (AdwViewStackPage *self)
@@ -1338,8 +1594,6 @@ adw_view_stack_page_get_visible (AdwViewStackPage *self)
  *
  * This is independent from the [property@Gtk.Widget:visible] property of
  * [property@ViewStackPage:child].
- *
- * Since: 1.0
  */
 void
 adw_view_stack_page_set_visible (AdwViewStackPage *self,
@@ -1366,8 +1620,6 @@ adw_view_stack_page_set_visible (AdwViewStackPage *self,
  * Creates a new `AdwViewStack`.
  *
  * Returns: the newly created `AdwViewStack`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_view_stack_new (void)
@@ -1383,8 +1635,6 @@ adw_view_stack_new (void)
  * Adds a child to @self.
  *
  * Returns: (transfer none): the [class@ViewStackPage] for @child
- *
- * Since: 1.0
  */
 AdwViewStackPage *
 adw_view_stack_add (AdwViewStack   *self,
@@ -1392,6 +1642,7 @@ adw_view_stack_add (AdwViewStack   *self,
 {
   g_return_val_if_fail (ADW_IS_VIEW_STACK (self), NULL);
   g_return_val_if_fail (GTK_IS_WIDGET (child), NULL);
+  g_return_val_if_fail (gtk_widget_get_parent (child) == NULL, NULL);
 
   return add_internal (self, child, NULL, NULL, NULL);
 }
@@ -1407,8 +1658,6 @@ adw_view_stack_add (AdwViewStack   *self,
  * The child is identified by the @name.
  *
  * Returns: (transfer none): the `AdwViewStackPage` for @child
- *
- * Since: 1.0
  */
 AdwViewStackPage *
 adw_view_stack_add_named (AdwViewStack   *self,
@@ -1417,6 +1666,7 @@ adw_view_stack_add_named (AdwViewStack   *self,
 {
   g_return_val_if_fail (ADW_IS_VIEW_STACK (self), NULL);
   g_return_val_if_fail (GTK_IS_WIDGET (child), NULL);
+  g_return_val_if_fail (gtk_widget_get_parent (child) == NULL, NULL);
 
   return add_internal (self, child, name, NULL, NULL);
 }
@@ -1434,8 +1684,6 @@ adw_view_stack_add_named (AdwViewStack   *self,
  * [class@ViewSwitcher] to represent @child, so it should be short.
  *
  * Returns: (transfer none): the `AdwViewStackPage` for @child
- *
- * Since: 1.0
  */
 AdwViewStackPage *
 adw_view_stack_add_titled (AdwViewStack   *self,
@@ -1445,6 +1693,7 @@ adw_view_stack_add_titled (AdwViewStack   *self,
 {
   g_return_val_if_fail (ADW_IS_VIEW_STACK (self), NULL);
   g_return_val_if_fail (GTK_IS_WIDGET (child), NULL);
+  g_return_val_if_fail (gtk_widget_get_parent (child) == NULL, NULL);
 
   return add_internal (self, child, name, title, NULL);
 }
@@ -1475,6 +1724,7 @@ adw_view_stack_add_titled_with_icon (AdwViewStack *self,
 {
   g_return_val_if_fail (ADW_IS_VIEW_STACK (self), NULL);
   g_return_val_if_fail (GTK_IS_WIDGET (child), NULL);
+  g_return_val_if_fail (gtk_widget_get_parent (child) == NULL, NULL);
 
   return add_internal (self, child, name, title, icon_name);
 }
@@ -1485,8 +1735,6 @@ adw_view_stack_add_titled_with_icon (AdwViewStack *self,
  * @child: the child to remove
  *
  * Removes a child widget from @self.
- *
- * Since: 1.0
  */
 void
 adw_view_stack_remove (AdwViewStack  *self,
@@ -1520,8 +1768,6 @@ adw_view_stack_remove (AdwViewStack  *self,
  * Gets the [class@ViewStackPage] object for @child.
  *
  * Returns: (transfer none): the page object for @child
- *
- * Since: 1.0
  */
 AdwViewStackPage *
 adw_view_stack_get_page (AdwViewStack  *self,
@@ -1541,8 +1787,6 @@ adw_view_stack_get_page (AdwViewStack  *self,
  * Finds the child with @name in @self.
  *
  * Returns: (transfer none) (nullable): the requested child
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_view_stack_get_child_by_name (AdwViewStack *self,
@@ -1565,8 +1809,6 @@ adw_view_stack_get_child_by_name (AdwViewStack *self,
  * Gets the currently visible child of @self, .
  *
  * Returns: (transfer none) (nullable): the visible child
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_view_stack_get_visible_child (AdwViewStack *self)
@@ -1582,8 +1824,6 @@ adw_view_stack_get_visible_child (AdwViewStack *self)
  * @child: a child of @self
  *
  * Makes @child the visible child of @self.
- *
- * Since: 1.0
  */
 void
 adw_view_stack_set_visible_child (AdwViewStack *self,
@@ -1613,8 +1853,6 @@ adw_view_stack_set_visible_child (AdwViewStack *self,
  * Returns the name of the currently visible child of @self.
  *
  * Returns: (transfer none) (nullable): the name of the visible child
- *
- * Since: 1.0
  */
 const char *
 adw_view_stack_get_visible_child_name (AdwViewStack *self)
@@ -1632,8 +1870,6 @@ adw_view_stack_get_visible_child_name (AdwViewStack *self)
  * Makes the child with @name visible.
  *
  * See [property@ViewStack:visible-child].
- *
- * Since: 1.0
  */
 void
 adw_view_stack_set_visible_child_name (AdwViewStack *self,
@@ -1665,8 +1901,6 @@ adw_view_stack_set_visible_child_name (AdwViewStack *self,
  * Gets whether @self is horizontally homogeneous.
  *
  * Returns: whether @self is horizontally homogeneous
- *
- * Since: 1.0
  */
 gboolean
 adw_view_stack_get_hhomogeneous (AdwViewStack *self)
@@ -1688,8 +1922,6 @@ adw_view_stack_get_hhomogeneous (AdwViewStack *self)
  *
  * If it's `FALSE`, the stack may change width when a different child becomes
  * visible.
- *
- * Since: 1.0
  */
 void
 adw_view_stack_set_hhomogeneous (AdwViewStack *self,
@@ -1717,8 +1949,6 @@ adw_view_stack_set_hhomogeneous (AdwViewStack *self,
  * Gets whether @self is vertically homogeneous.
  *
  * Returns: whether @self is vertically homogeneous
- *
- * Since: 1.0
  */
 gboolean
 adw_view_stack_get_vhomogeneous (AdwViewStack *self)
@@ -1740,8 +1970,6 @@ adw_view_stack_get_vhomogeneous (AdwViewStack *self)
  *
  * If it's `FALSE`, the stack may change height when a different child becomes
  * visible.
- *
- * Since: 1.0
  */
 void
 adw_view_stack_set_vhomogeneous (AdwViewStack *self,
@@ -1773,8 +2001,6 @@ adw_view_stack_set_vhomogeneous (AdwViewStack *self,
  * page.
  *
  * Returns: (transfer full): a `GtkSelectionModel` for the stack's children
- *
- * Since: 1.0
  */
 GtkSelectionModel *
 adw_view_stack_get_pages (AdwViewStack *self)
