@@ -5,9 +5,8 @@
  */
 
 #include "config.h"
-#include "adw-window-title.h"
 
-#include "adw-macros-private.h"
+#include "adw-window-title.h"
 
 /**
  * AdwWindowTitle:
@@ -25,8 +24,6 @@
  * ## CSS nodes
  *
  * `AdwWindowTitle` has a single CSS node with name `windowtitle`.
- *
- * Since: 1.0
  */
 
 enum {
@@ -100,10 +97,7 @@ adw_window_title_set_property (GObject      *object,
 static void
 adw_window_title_dispose (GObject *object)
 {
-  AdwWindowTitle *self = ADW_WINDOW_TITLE (object);
-
-  if (self->box)
-    gtk_widget_unparent (GTK_WIDGET (self->box));
+  gtk_widget_dispose_template (GTK_WIDGET (object), ADW_TYPE_WINDOW_TITLE);
 
   G_OBJECT_CLASS (adw_window_title_parent_class)->dispose (object);
 }
@@ -125,31 +119,23 @@ adw_window_title_class_init (AdwWindowTitleClass *klass)
    *
    * The title typically identifies the current view or content item, and
    * generally does not use the application name.
-   *
-   * Since: 1.0
    */
   props[PROP_TITLE] =
-    g_param_spec_string ("title",
-                         "Title",
-                         "The title to display",
+    g_param_spec_string ("title", NULL, NULL,
                          "",
-                         G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
    * AdwWindowTitle:subtitle: (attributes org.gtk.Property.get=adw_window_title_get_subtitle org.gtk.Property.set=adw_window_title_set_subtitle)
    *
    * The subtitle to display.
    *
-   * The subtitle should give a user additional details.
-   *
-   * Since: 1.0
+   * The subtitle should give the user additional details.
    */
   props[PROP_SUBTITLE] =
-    g_param_spec_string ("subtitle",
-                         "Subtitle",
-                         "The subtitle to display",
+    g_param_spec_string ("subtitle", NULL, NULL,
                          "",
-                         G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
@@ -171,8 +157,6 @@ adw_window_title_class_init (AdwWindowTitleClass *klass)
  * Creates a new `AdwWindowTitle`.
  *
  * Returns: the newly created `AdwWindowTitle`
- *
- * Since: 1.0
  */
 GtkWidget *
 adw_window_title_new (const char *title,
@@ -191,8 +175,6 @@ adw_window_title_new (const char *title,
  * Gets the title of @self.
  *
  * Returns: the title
- *
- * Since: 1.0
  */
 const char *
 adw_window_title_get_title (AdwWindowTitle *self)
@@ -209,7 +191,8 @@ adw_window_title_get_title (AdwWindowTitle *self)
  *
  * Sets the title of @self.
  *
- * Since: 1.0
+ * The title typically identifies the current view or content item, and
+ * generally does not use the application name.
  */
 void
 adw_window_title_set_title (AdwWindowTitle *self,
@@ -234,8 +217,6 @@ adw_window_title_set_title (AdwWindowTitle *self,
  * Gets the subtitle of @self.
  *
  * Returns: the subtitle
- *
- * Since: 1.0
  */
 const char *
 adw_window_title_get_subtitle (AdwWindowTitle *self)
@@ -252,7 +233,7 @@ adw_window_title_get_subtitle (AdwWindowTitle *self)
  *
  * Sets the subtitle of @self.
  *
- * Since: 1.0
+ * The subtitle should give the user additional details.
  */
 void
 adw_window_title_set_subtitle (AdwWindowTitle *self,
